#!/usr/bin/env python
import pytest

import numpy as np
import pandas as pd
import pandas.compat as compat

import sklearn.datasets as datasets
import sklearn.metrics as metrics

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestMetricsCommon(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.metrics.make_scorer, metrics.make_scorer)

    def test_make_scorer(self):
        df = pdml.ModelFrame([])

        result = df.metrics.make_scorer(metrics.fbeta_score, beta=2)
        expected = metrics.make_scorer(metrics.fbeta_score, beta=2)

        self.assertEqual(result._kwargs, expected._kwargs)
        self.assertEqual(result._sign, expected._sign)
        self.assertEqual(result._score_func, expected._score_func)


class TestClassificationMetrics(tm.TestCase):

    def setup_method(self):
        import sklearn.svm as svm
        digits = datasets.load_digits()
        self.data = digits.data
        self.target = digits.target
        self.df = pdml.ModelFrame(digits)

        estimator1 = self.df.svm.LinearSVC(C=1.0, random_state=self.random_state)
        self.df.fit(estimator1)

        estimator2 = svm.LinearSVC(C=1.0, random_state=self.random_state)
        estimator2.fit(self.data, self.target)
        self.pred = estimator2.predict(self.data)
        self.decision = estimator2.decision_function(self.data)

        # argument for classification reports
        self.labels = np.array([9, 8, 7, 6, 5, 4, 3, 2, 1, 0])

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.metrics.make_scorer, metrics.make_scorer)

    def test_make_scorer(self):
        df = pdml.ModelFrame([])

        result = df.metrics.make_scorer(metrics.fbeta_score, beta=2)
        expected = metrics.make_scorer(metrics.fbeta_score, beta=2)

        self.assertEqual(result._kwargs, expected._kwargs)
        self.assertEqual(result._sign, expected._sign)
        self.assertEqual(result._score_func, expected._score_func)

    def test_accuracy_score(self):
        result = self.df.metrics.accuracy_score()
        expected = metrics.accuracy_score(self.target, self.pred)
        self.assertEqual(result, expected)

        result = self.df.metrics.accuracy_score(normalize=False)
        expected = metrics.accuracy_score(self.target, self.pred,
                                          normalize=False)
        self.assertEqual(result, expected)

    def test_auc(self):
        # Tested in:
        #
        # - test_precision_recall_curve
        # - test_roc_curve
        pass

    def test_average_precision_score(self):
        with pytest.raises(ValueError, match='multiclass format is not supported'):
            self.df.metrics.average_precision_score()

    def test_classification_report(self):
        result = self.df.metrics.classification_report()
        expected = metrics.classification_report(self.target, self.pred)
        self.assertEqual(result, expected)

        result = self.df.metrics.classification_report(labels=self.labels)
        expected = metrics.classification_report(self.target, self.pred, labels=self.labels)
        self.assertEqual(result, expected)

    def test_confusion_matrix(self):
        result = self.df.metrics.confusion_matrix()
        expected = metrics.confusion_matrix(self.target, self.pred)
        self.assertTrue(isinstance(result, pdml.ModelFrame))
        tm.assert_numpy_array_equal(result.values, expected)

        result = self.df.metrics.confusion_matrix(labels=self.labels)
        expected = metrics.confusion_matrix(self.target, self.pred, labels=self.labels)
        self.assertTrue(isinstance(result, pdml.ModelFrame))
        tm.assert_numpy_array_equal(result.values, expected)

    def test_f1_score(self):
        result = self.df.metrics.f1_score(average='weighted')
        expected = metrics.f1_score(self.target, self.pred, average='weighted')
        self.assertAlmostEqual(result, expected)

        result = self.df.metrics.f1_score(average=None)
        expected = metrics.f1_score(self.target, self.pred, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    @pytest.mark.parametrize("average", ['weighted', 'macro'])
    def test_fbeta_score_averaging(self, average):
        result = self.df.metrics.fbeta_score(beta=0.5, average=average)
        expected = metrics.fbeta_score(self.target, self.pred, beta=0.5, average=average)
        self.assertEqual(result, expected)

    def test_fbeta_score(self):
        result = self.df.metrics.fbeta_score(beta=0.5, average=None)
        expected = metrics.fbeta_score(self.target, self.pred, beta=0.5, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_hamming_loss(self):
        result = self.df.metrics.hamming_loss()
        expected = metrics.hamming_loss(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_hinge_loss(self):
        result = self.df.metrics.hinge_loss()
        expected = metrics.hinge_loss(self.target, self.decision)
        self.assertEqual(result, expected)

    def test_jaccard_similarity_score(self):
        result = self.df.metrics.jaccard_similarity_score()
        expected = metrics.jaccard_similarity_score(self.target, self.pred)
        self.assertEqual(result, expected)

        result = self.df.metrics.jaccard_similarity_score(normalize=False)
        expected = metrics.jaccard_similarity_score(self.target, self.pred, normalize=False)
        self.assertEqual(result, expected)

    def test_log_loss(self):
        msg = "class <class 'sklearn.svm.classes.LinearSVC'> doesn't have predict_proba method"
        with pytest.raises(ValueError, match=msg):
            self.df.metrics.log_loss()

    def test_matthews_corrcoef(self):
        if not pdml.compat._SKLEARN_ge_019:
            with pytest.raises(ValueError, match='multiclass is not supported'):
                self.df.metrics.matthews_corrcoef()

    def test_precision_recall_curve(self):
        results = self.df.metrics.precision_recall_curve()
        for key, result in compat.iteritems(results):
            expected = metrics.precision_recall_curve(self.target, self.decision[:, key],
                                                      pos_label=key)
            self.assert_numpy_array_almost_equal(result[0], expected[0])
            self.assert_numpy_array_almost_equal(result[1], expected[1])
            self.assert_numpy_array_almost_equal(result[2], expected[2])

    def test_precision_recall_fscore_support(self):
        result = self.df.metrics.precision_recall_fscore_support()
        expected = metrics.precision_recall_fscore_support(self.target, self.pred)
        self.assert_numpy_array_almost_equal(result['precision'].values, expected[0])
        self.assert_numpy_array_almost_equal(result['recall'].values, expected[1])
        self.assert_numpy_array_almost_equal(result['f1-score'].values, expected[2])
        self.assert_numpy_array_almost_equal(result['support'].values, expected[3])

        expected = pd.Index(['precision', 'recall', 'f1-score', 'support'])
        tm.assert_index_equal(result.columns, expected)

    def test_precision_score(self):
        result = self.df.metrics.precision_score(average='weighted')
        expected = metrics.precision_score(self.target, self.pred, average='weighted')
        self.assertEqual(result, expected)

        result = self.df.metrics.precision_score(average=None)
        expected = metrics.precision_score(self.target, self.pred, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_recall_score(self):
        result = self.df.metrics.recall_score(average='weighted')
        expected = metrics.recall_score(self.target, self.pred, average='weighted')
        self.assertEqual(result, expected)

        result = self.df.metrics.recall_score(average=None)
        expected = metrics.recall_score(self.target, self.pred, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_roc_auc_score(self):
        with pytest.raises(ValueError, match='multiclass format is not supported'):
            self.df.metrics.roc_auc_score()

    def test_roc_curve(self):
        results = self.df.metrics.roc_curve()
        for key, result in compat.iteritems(results):
            expected = metrics.roc_curve(self.target, self.decision[:, key],
                                         pos_label=key)
            self.assert_numpy_array_almost_equal(result[0], expected[0])
            self.assert_numpy_array_almost_equal(result[1], expected[1])
            self.assert_numpy_array_almost_equal(result[2], expected[2])

    def test_zero_one_loss(self):
        result = self.df.metrics.zero_one_loss()
        expected = metrics.zero_one_loss(self.target, self.pred)
        self.assertEqual(result, expected)


class TestClassificationMetrics2Classes(TestClassificationMetrics):

    def setup_method(self):
        import sklearn.svm as svm
        # 2 class
        iris = datasets.load_iris()
        self.data = iris.data[0:100]
        self.target = [1 if t == 1 else -1 for t in iris.target[0:100]]
        self.df = pdml.ModelFrame(self.data, target=self.target)

        estimator1 = self.df.svm.SVC(probability=True, random_state=self.random_state)
        self.df.fit(estimator1)
        self.df.predict(estimator1)

        estimator2 = svm.SVC(probability=True, random_state=self.random_state)
        estimator2.fit(self.data, self.target)
        self.pred = estimator2.predict(self.data)
        self.proba = estimator2.predict_proba(self.data)
        self.decision = estimator2.decision_function(self.data)

        # argument for classification reports
        self.labels = np.array([1, -1])

    def test_average_precision_score(self):
        result = self.df.metrics.average_precision_score(average='weighted')
        expected = metrics.average_precision_score(self.target, self.decision,
                                                   average='weighted')
        self.assertEqual(result, expected)
        curve, _, _ = self.df.metrics.precision_recall_curve()
        self.assertEqual(result, curve.mean())

        result = self.df.metrics.average_precision_score(average=None)

        expected = metrics.average_precision_score(self.target, self.decision, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_log_loss(self):
        result = self.df.metrics.log_loss()
        expected = metrics.log_loss(self.target, self.proba)
        self.assertEqual(result, expected)

    def test_matthews_corrcoef(self):
        result = self.df.metrics.matthews_corrcoef()
        expected = metrics.matthews_corrcoef(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_precision_recall_curve(self):
        result = self.df.metrics.precision_recall_curve()
        expected = metrics.precision_recall_curve(self.target, self.decision)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])
        self.assert_numpy_array_almost_equal(result[2], expected[2])

        # test average_precision_score is identical as auc
        result = self.df.metrics.auc(kind='precision_recall_curve')
        expected = metrics.auc(expected[1], expected[0])
        self.assertEqual(result, expected)

        expected = metrics.average_precision_score(self.target, self.decision)
        self.assertEqual(result, expected)

    def test_roc_auc_score(self):
        result = self.df.metrics.roc_auc_score(average='weighted')
        expected = metrics.roc_auc_score(self.target, self.decision, average='weighted')
        self.assertEqual(result, expected)

        result = self.df.metrics.roc_auc_score(average=None)
        expected = metrics.roc_auc_score(self.target, self.decision, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_roc_curve(self):
        result = self.df.metrics.roc_curve()
        expected = metrics.roc_curve(self.target, self.decision)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])
        self.assert_numpy_array_almost_equal(result[2], expected[2])

        # test roc_auc_score is identical as auc
        result = self.df.metrics.auc(kind='roc')
        expected = metrics.auc(expected[0], expected[1])
        self.assertEqual(result, expected)

        expected = metrics.roc_auc_score(self.target, self.decision)
        self.assertEqual(result, expected)


class TestClassificationMetrics3Classes(TestClassificationMetrics):

    def setup_method(self):
        import sklearn.svm as svm
        # 2 class
        iris = datasets.load_iris()
        self.data = iris.data
        self.target = iris.target
        self.df = pdml.ModelFrame(self.data, target=self.target)

        estimator1 = svm.SVC(probability=True, random_state=self.random_state)
        self.df.fit(estimator1)
        self.df.predict(estimator1)

        estimator2 = svm.SVC(probability=True, random_state=self.random_state)
        estimator2.fit(self.data, self.target)
        self.pred = estimator2.predict(self.data)
        self.proba = estimator2.predict_proba(self.data)
        self.decision = estimator2.decision_function(self.data)

        # argument for classification reports
        self.labels = np.array([2, 1, 0])

    def test_log_loss(self):
        result = self.df.metrics.log_loss()
        expected = metrics.log_loss(self.target, self.proba)
        self.assertEqual(result, expected)

    def test_average_precision_score(self):
        with pytest.raises(ValueError, match='multiclass format is not supported'):
            self.df.metrics.average_precision_score()


class TestClassificationMetrics3ClassesMultiTargets(TestClassificationMetrics):

    def setup_method(self):
        import sklearn.svm as svm
        import sklearn.preprocessing as pp
        from sklearn.multiclass import OneVsRestClassifier

        # 2 class
        iris = datasets.load_iris()
        self.data = iris.data
        self.target = pp.LabelBinarizer().fit_transform(iris.target)
        self.df = pdml.ModelFrame(self.data, target=self.target)
        self.assertEqual(self.df.shape, (150, 7))

        svc1 = svm.SVC(probability=True, random_state=self.random_state)
        estimator1 = OneVsRestClassifier(svc1)
        self.df.fit(estimator1)
        self.df.predict(estimator1)
        self.assertTrue(isinstance(self.df.predicted, pdml.ModelFrame))

        svc2 = svm.SVC(probability=True, random_state=self.random_state)
        estimator2 = OneVsRestClassifier(svc2)
        estimator2.fit(self.data, self.target)
        self.pred = estimator2.predict(self.data)
        self.proba = estimator2.predict_proba(self.data)
        self.decision = estimator2.decision_function(self.data)

        # argument for classification reports
        self.labels = np.array([2, 1, 0])

    def test_average_precision_score(self):
        result = self.df.metrics.average_precision_score(average='weighted')
        expected = metrics.average_precision_score(self.target, self.decision,
                                                   average='weighted')
        self.assertAlmostEqual(result, expected)
        # curve, _, _ = self.df.metrics.precision_recall_curve()
        # self.assertEqual(result, curve.mean())

        result = self.df.metrics.average_precision_score(average=None)

        expected = metrics.average_precision_score(self.target, self.decision, average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_confusion_matrix(self):
        with pytest.raises(ValueError, match='multilabel-indicator is not supported'):
            self.df.metrics.confusion_matrix()

    def test_hinge_loss(self):
        return

    def test_log_loss(self):
        result = self.df.metrics.log_loss()
        expected = metrics.log_loss(self.target, self.proba)
        self.assertEqual(result, expected)

    def test_matthews_corrcoef(self):
        msg = 'multilabel-indicator is not supported'
        with pytest.raises(ValueError, match=msg):
            self.df.metrics.matthews_corrcoef()

    def test_precision_recall_curve(self):
        results = self.df.metrics.precision_recall_curve()
        for key, result in compat.iteritems(results):
            expected = metrics.precision_recall_curve(self.target[:, key],
                                                      self.decision[:, key],
                                                      pos_label=key)
            self.assert_numpy_array_almost_equal(result[0], expected[0])
            self.assert_numpy_array_almost_equal(result[1], expected[1])
            self.assert_numpy_array_almost_equal(result[2], expected[2])

    def test_roc_auc_score(self):
        result = self.df.metrics.roc_auc_score(average='weighted')
        expected = metrics.roc_auc_score(self.target, self.decision,
                                         average='weighted')
        self.assertEqual(result, expected)

        result = self.df.metrics.roc_auc_score(average=None)
        expected = metrics.roc_auc_score(self.target, self.decision,
                                         average=None)
        self.assertTrue(isinstance(result, pdml.ModelSeries))
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_roc_curve(self):
        results = self.df.metrics.roc_curve()
        for key, result in compat.iteritems(results):
            expected = metrics.roc_curve(self.target[:, key], self.decision[:, key],
                                         pos_label=key)
            self.assert_numpy_array_almost_equal(result[0], expected[0])
            self.assert_numpy_array_almost_equal(result[1], expected[1])
            self.assert_numpy_array_almost_equal(result[2], expected[2])


class TestClassificationMetricsExamples(tm.TestCase):

    def test_hinge_loss(self):
        X = [[0], [1]]
        y = [-1, 1]
        df = pdml.ModelFrame(X, target=y)
        est = df.svm.LinearSVC(random_state=self.random_state)
        df.fit(est)

        test_df = pdml.ModelFrame([[-2], [3], [0.5]], target=[-1, 1, 1])
        decisions = test_df.decision_function(est)
        expected = np.array([[-2.18177943769], [2.36355887537], [0.0908897188437]])
        self.assert_numpy_array_almost_equal(decisions.values, expected)
        self.assertAlmostEqual(test_df.metrics.hinge_loss(), 0.30303676038544253)


class TestRegressionMetrics(tm.TestCase):

    def setup_method(self):
        from sklearn import linear_model
        data = [[0, 0], [1, 1], [2, 2]]
        self.target = [0, 1, 2]

        self.df = pdml.ModelFrame(data=data, target=self.target)

        estimator1 = linear_model.LinearRegression()
        self.df.fit(estimator1)
        self.df.predict(estimator1)

        estimator2 = linear_model.LinearRegression()

        estimator2.fit(data, self.target)
        self.pred = estimator2.predict(data)

    def test_explained_variance_score(self):
        result = self.df.metrics.explained_variance_score()
        expected = metrics.explained_variance_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_mean_absolute_error(self):
        result = self.df.metrics.mean_absolute_error()
        expected = metrics.mean_absolute_error(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_mean_squared_error(self):
        result = self.df.metrics.mean_squared_error()
        expected = metrics.mean_squared_error(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_r2_score(self):
        result = self.df.metrics.r2_score()
        expected = metrics.r2_score(self.target, self.pred)
        self.assertEqual(result, expected)


class TestClusteringMetrics(tm.TestCase):

    def setup_method(self):
        from sklearn import cluster
        iris = datasets.load_iris()
        self.data = iris.data
        self.target = iris.target

        self.df = pdml.ModelFrame(iris)
        estimator1 = cluster.KMeans(3, random_state=self.random_state)

        self.df.fit(estimator1)
        self.df.predict(estimator1)

        estimator2 = cluster.KMeans(3, random_state=self.random_state)
        estimator2.fit(iris.data, self.target)
        self.pred = estimator2.predict(iris.data)

    def test_adjusted_mutual_info_score(self):
        result = self.df.metrics.adjusted_mutual_info_score()
        expected = metrics.adjusted_mutual_info_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_adjusted_rand_score(self):
        result = self.df.metrics.adjusted_rand_score()
        expected = metrics.adjusted_rand_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_completeness_score(self):
        result = self.df.metrics.completeness_score()
        expected = metrics.completeness_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_homogeneity_completeness_v_measure(self):
        result = self.df.metrics.homogeneity_completeness_v_measure()
        expected = metrics.homogeneity_completeness_v_measure(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_homogeneity_score(self):
        result = self.df.metrics.homogeneity_score()
        expected = metrics.homogeneity_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_mutual_info_score(self):
        result = self.df.metrics.mutual_info_score()
        expected = metrics.mutual_info_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_normalized_mutual_info_score(self):
        result = self.df.metrics.normalized_mutual_info_score()
        expected = metrics.normalized_mutual_info_score(self.target, self.pred)
        self.assertEqual(result, expected)

    def test_silhouette_score(self):
        result = self.df.metrics.silhouette_score()
        expected = metrics.silhouette_score(self.data, self.pred)
        self.assertAlmostEqual(result, expected)

    def test_silhouette_samples(self):
        result = self.df.metrics.silhouette_samples()
        expected = metrics.silhouette_samples(self.data, self.pred)

        self.assertTrue(isinstance(result, pdml.ModelSeries))
        tm.assert_index_equal(result.index, self.df.index)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_v_measure_score(self):
        result = self.df.metrics.v_measure_score()
        expected = metrics.v_measure_score(self.target, self.pred)
        self.assertEqual(result, expected)



#!/usr/bin/env python

from pandas_ml.skaccessors.base import _maybe_sklearn_data                        # noqa
from pandas_ml.skaccessors.cluster import ClusterMethods                          # noqa
from pandas_ml.skaccessors.covariance import CovarianceMethods                    # noqa
from pandas_ml.skaccessors.cross_decomposition import CrossDecompositionMethods   # noqa
from pandas_ml.skaccessors.decomposition import DecompositionMethods              # noqa
from pandas_ml.skaccessors.ensemble import EnsembleMethods                        # noqa
from pandas_ml.skaccessors.feature_extraction import FeatureExtractionMethods     # noqa
from pandas_ml.skaccessors.feature_selection import FeatureSelectionMethods       # noqa
from pandas_ml.skaccessors.gaussian_process import GaussianProcessMethods         # noqa
from pandas_ml.skaccessors.isotonic import IsotonicMethods                        # noqa
from pandas_ml.skaccessors.linear_model import LinearModelMethods                 # noqa
from pandas_ml.skaccessors.manifold import ManifoldMethods                        # noqa
from pandas_ml.skaccessors.metrics import MetricsMethods                          # noqa
from pandas_ml.skaccessors.model_selection import ModelSelectionMethods           # noqa
from pandas_ml.skaccessors.neighbors import NeighborsMethods                      # noqa
from pandas_ml.skaccessors.pipeline import PipelineMethods                        # noqa
from pandas_ml.skaccessors.preprocessing import PreprocessingMethods              # noqa
from pandas_ml.skaccessors.svm import SVMMethods                                  # noqa



#!/usr/bin/env python

import pandas as pd

from pandas_ml.core.accessor import _AccessorMethods, _attach_methods


class SeabornMethods(_AccessorMethods):
    """Accessor to ``sklearn.cluster``."""

    _module_name = 'seaborn'
    _module_attrs = ['palplot', 'set', 'axes_style', 'plotting_context',
                     'set_context', 'set_color_codes', 'reset_defaults',
                     'reset_orig', 'set_palette', 'color_palette',
                     'husl_palette', 'hls_palette', 'cubehelix_palette',
                     'dark_palette', 'light_palette', 'diverging_palette',
                     'blend_palette', 'xkcd_palette', 'crayon_palette',
                     'mpl_palette', 'choose_colorbrewer_palette',
                     'choose_cubehelix_palette', 'choose_light_palette',
                     'choose_dark_palette', 'choose_diverging_palette',
                     'despine', 'desaturate', 'saturate', 'set_hls_values',
                     'ci_to_errsize', 'axlabel']

    def _maybe_target_name(self, value, key):
        if value is None:
            if not self._df.has_target():
                msg = ("{key} can't be ommitted when ModelFrame doesn't have "
                       "target column")
                raise ValueError(msg.format(key=key))
            elif self._df.has_multi_targets():
                msg = ("{key} can't be ommitted when ModelFrame has multiple "
                       "target columns")
                raise ValueError(msg.format(key=key))
            value = self._df.target_name
        return value

    def _maybe_target_series(self, value, key):
        if value is None:
            if not self._df.has_target():
                msg = ("{key} can't be ommitted when ModelFrame doesn't have "
                       "target column")
                raise ValueError(msg.format(key=key))
            elif self._df.has_multi_targets():
                msg = ("{key} can't be ommitted when ModelFrame has multiple "
                       "target columns")
                raise ValueError(msg.format(key=key))
            value = self._df.target

        elif not pd.api.types.is_list_like(value):
            value = self._df[value]
        return value

    # Axis grids

    def FacetGrid(self, row=None, col=None, *args, **kwargs):
        return self._module.FacetGrid(data=self._df, row=row, col=col,
                                      *args, **kwargs)

    def PairGrid(self, *args, **kwargs):
        return self._module.PairGrid(data=self._df, *args, **kwargs)

    def JointGrid(self, x, y, *args, **kwargs):
        return self._module.JointGrid(x, y, data=self._df, *args, **kwargs)

    # Distribution plots

    def distplot(self, a=None, *args, **kwargs):
        """
        Call ``seaborn.distplot`` using automatic mapping.

        - ``a``: ``ModelFrame.target``
        """
        a = self._maybe_target_series(a, key='a')
        return self._module.distplot(a, *args, **kwargs)

    def rugplot(self, a=None, *args, **kwargs):
        """
        Call ``seaborn.rugplot`` using automatic mapping.

        - ``a``: ``ModelFrame.target``
        """
        a = self._maybe_target_series(a, key='a')
        return self._module.rugplot(a, *args, **kwargs)

    def kdeplot(self, data=None, data2=None, *args, **kwargs):
        """
        Call ``seaborn.kdeplot`` using automatic mapping.

        - ``data``: ``ModelFrame.target``
        """
        data = self._maybe_target_series(data, key='data')

        if data2 is not None:
            if not pd.api.types.is_list_like(data2):
                data2 = self._df[data2]
        return self._module.kdeplot(data, data2=data2, *args, **kwargs)

    # Regression plots

    def interactplot(self, x1, x2, y=None, *args, **kwargs):
        """
        Call ``seaborn.interactplot`` using automatic mapping.

        - ``data``: ``ModelFrame``
        - ``y``: ``ModelFrame.target_name``
        """

        y = self._maybe_target_name(y, key='y')
        return self._module.interactplot(x1, x2, y, data=self._df,
                                         *args, **kwargs)

    def coefplot(self, formula, *args, **kwargs):
        """
        Call ``seaborn.coefplot`` using automatic mapping.

        - ``data``: ``ModelFrame``
        """
        return self._module.coefplot(formula, data=self._df, *args, **kwargs)

    # Categorical plots

    def countplot(self, x=None, y=None, *args, **kwargs):
        """
        Call ``seaborn.countplot`` using automatic mapping.

        - ``data``: ``ModelFrame``
        - ``y``: ``ModelFrame.target_name``
        """
        if x is None and y is None:
            x = self._maybe_target_name(x, key='x')
        return self._module.countplot(x, y, data=self._df, *args, **kwargs)

    # Matrix plots

    def heatmap(self, *args, **kwargs):
        """
        Call ``seaborn.heatmap`` using automatic mapping.

        - ``data``: ``ModelFrame``
        """
        return self._module.heatmap(data=self._df, *args, **kwargs)

    def clustermap(self, *args, **kwargs):
        """
        Call ``seaborn.clustermap`` using automatic mapping.

        - ``data``: ``ModelFrame``
        """
        return self._module.clustermap(data=self._df, *args, **kwargs)

    # Timeseries plots

    def tsplot(self, *args, **kwargs):
        """
        Call ``seaborn.tsplot`` using automatic mapping.

        - ``data``: ``ModelFrame``
        """
        return self._module.tsplot(data=self._df, *args, **kwargs)


def _wrap_xy_plot(func, func_name):
    """
    Wrapper for plotting with x, y, data
    """
    def f(self, x, y=None, *args, **kwargs):
        y = self._maybe_target_name(y, key='y')
        return func(x, y, data=self._df, *args, **kwargs)

    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``data``: ``ModelFrame``
        - ``y``: ``ModelFrame.target_name``
        """ % func_name)
    return f


def _wrap_categorical_plot(func, func_name):
    """
    Wrapper for categorical, x and y may be optional
    """
    def f(self, y=None, x=None, *args, **kwargs):

        if x is not None and y is None:
            y = self._maybe_target_name(y, key='y')

        elif x is None and y is not None:
            x = self._maybe_target_name(x, key='x')
        return func(x, y, data=self._df, *args, **kwargs)

    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping. If you omit x


        - ``data``: ``ModelFrame``
        - ``x``: ``ModelFrame.target_name``
        """ % func_name)
    return f


def _wrap_data_plot(func, func_name):
    """
    Wrapper for plotting with data
    """
    def f(self, *args, **kwargs):
        return func(data=self._df, *args, **kwargs)

    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``data``: ``ModelFrame``
        """ % func_name)
    return f


_xy_plots = ['jointplot', 'lmplot', 'regplot', 'residplot']
_attach_methods(SeabornMethods, _wrap_xy_plot, _xy_plots)

_categorical_plots = ['factorplot', 'boxplot', 'violinplot', 'stripplot',
                      'pointplot', 'barplot']
_attach_methods(SeabornMethods, _wrap_categorical_plot, _categorical_plots)

_data_plots = ['pairplot']
_attach_methods(SeabornMethods, _wrap_data_plot, _data_plots)



#!/usr/bin/env python

import sklearn.datasets as datasets
try:
    import sklearn.model_selection as ms
except ImportError:
    pass
import sklearn.svm as svm
import sklearn.naive_bayes as nb

import numpy as np
import pandas as pd
import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestModelSelection(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])

        # Splitter Classes
        self.assertIs(df.model_selection.KFold, ms.KFold)
        self.assertIs(df.model_selection.GroupKFold, ms.GroupKFold)
        self.assertIs(df.model_selection.StratifiedKFold, ms.StratifiedKFold)

        self.assertIs(df.model_selection.LeaveOneGroupOut, ms.LeaveOneGroupOut)
        self.assertIs(df.model_selection.LeavePGroupsOut, ms.LeavePGroupsOut)
        self.assertIs(df.model_selection.LeaveOneOut, ms.LeaveOneOut)
        self.assertIs(df.model_selection.LeavePOut, ms.LeavePOut)

        self.assertIs(df.model_selection.ShuffleSplit, ms.ShuffleSplit)
        self.assertIs(df.model_selection.GroupShuffleSplit,
                      ms.GroupShuffleSplit)
        # self.assertIs(df.model_selection.StratifiedShuffleSplit,
        #               ms.StratifiedShuffleSplit)
        self.assertIs(df.model_selection.PredefinedSplit, ms.PredefinedSplit)
        self.assertIs(df.model_selection.TimeSeriesSplit, ms.TimeSeriesSplit)

        # Splitter Functions

        # Hyper-parameter optimizers
        self.assertIs(df.model_selection.GridSearchCV, ms.GridSearchCV)
        self.assertIs(df.model_selection.RandomizedSearchCV, ms.RandomizedSearchCV)
        self.assertIs(df.model_selection.ParameterGrid, ms.ParameterGrid)
        self.assertIs(df.model_selection.ParameterSampler, ms.ParameterSampler)

        # Model validation

    def test_objectmapper_abbr(self):
        df = pdml.ModelFrame([])

        # Splitter Classes
        self.assertIs(df.ms.KFold, ms.KFold)
        self.assertIs(df.ms.GroupKFold, ms.GroupKFold)
        self.assertIs(df.ms.StratifiedKFold, ms.StratifiedKFold)

        self.assertIs(df.ms.LeaveOneGroupOut, ms.LeaveOneGroupOut)
        self.assertIs(df.ms.LeavePGroupsOut, ms.LeavePGroupsOut)
        self.assertIs(df.ms.LeaveOneOut, ms.LeaveOneOut)
        self.assertIs(df.ms.LeavePOut, ms.LeavePOut)

        self.assertIs(df.ms.ShuffleSplit, ms.ShuffleSplit)
        self.assertIs(df.ms.GroupShuffleSplit,
                      ms.GroupShuffleSplit)
        # self.assertIs(df.ms.StratifiedShuffleSplit,
        #               ms.StratifiedShuffleSplit)
        self.assertIs(df.ms.PredefinedSplit, ms.PredefinedSplit)
        self.assertIs(df.ms.TimeSeriesSplit, ms.TimeSeriesSplit)

        # Splitter Functions

        # Hyper-parameter optimizers
        self.assertIs(df.ms.GridSearchCV, ms.GridSearchCV)
        self.assertIs(df.ms.RandomizedSearchCV, ms.RandomizedSearchCV)
        self.assertIs(df.ms.ParameterGrid, ms.ParameterGrid)
        self.assertIs(df.ms.ParameterSampler, ms.ParameterSampler)

        # Model validation


class TestSplitter(tm.TestCase):

    def test_iterate(self):
        df = pdml.ModelFrame(datasets.load_iris())
        kf = df.model_selection.KFold(4, random_state=self.random_state)

        with tm.assert_produces_warning(FutureWarning):
            gen = df.model_selection.iterate(kf)

        for train_df, test_df in gen:
            self.assertIsInstance(train_df, pdml.ModelFrame)
            self.assertIsInstance(test_df, pdml.ModelFrame)
            tm.assert_index_equal(df.columns, train_df.columns)
            tm.assert_index_equal(df.columns, test_df.columns)

            self.assertEqual(len(df), len(train_df) + len(test_df))

    def test_split(self):
        df = pdml.ModelFrame(datasets.load_iris())
        kf = df.model_selection.KFold(4, random_state=self.random_state)

        gen = df.model_selection.split(kf)

        for train_df, test_df in gen:
            self.assertIsInstance(train_df, pdml.ModelFrame)
            self.assertIsInstance(test_df, pdml.ModelFrame)
            tm.assert_index_equal(df.columns, train_df.columns)
            tm.assert_index_equal(df.columns, test_df.columns)

            self.assertEqual(len(df), len(train_df) + len(test_df))

    def test_split_keep_index(self):
        df = pdml.ModelFrame({'A': [1, 2, 3, 4, 5, 6, 7, 8],
                              'B': [1, 2, 3, 4, 5, 6, 7, 8]},
                             index='a b c d e f g h'.split(' '))
        kf = df.model_selection.KFold(3, random_state=self.random_state)
        folded = [f for f in df.model_selection.split(kf)]
        self.assertEqual(len(folded), 3)
        tm.assert_frame_equal(folded[0][0], df.iloc[3:, :])
        tm.assert_frame_equal(folded[0][1], df.iloc[:3, :])
        tm.assert_frame_equal(folded[1][0], df.iloc[[0, 1, 2, 6, 7], :])
        tm.assert_frame_equal(folded[1][1], df.iloc[3:6, :])
        tm.assert_frame_equal(folded[2][0], df.iloc[:6, :])
        tm.assert_frame_equal(folded[2][1], df.iloc[6:, :])

        folded = [f for f in df.model_selection.split(kf, reset_index=True)]
        self.assertEqual(len(folded), 3)
        tm.assert_frame_equal(folded[0][0],
                              df.iloc[3:, :].reset_index(drop=True))
        tm.assert_frame_equal(folded[0][1],
                              df.iloc[:3, :].reset_index(drop=True))
        tm.assert_frame_equal(folded[1][0],
                              df.iloc[[0, 1, 2, 6, 7], :].reset_index(drop=True))
        tm.assert_frame_equal(folded[1][1],
                              df.iloc[3:6, :].reset_index(drop=True))
        tm.assert_frame_equal(folded[2][0],
                              df.iloc[:6, :].reset_index(drop=True))
        tm.assert_frame_equal(folded[2][1],
                              df.iloc[6:, :].reset_index(drop=True))

    def test_train_test_split(self):

        df = pdml.ModelFrame(datasets.load_digits())
        self.assertIsInstance(df, pdml.ModelFrame)

        train_df, test_df = df.model_selection.train_test_split()
        tm.assert_index_equal(df.columns, train_df.columns)
        tm.assert_index_equal(df.columns, test_df.columns)

        self.assertEqual(len(df), len(train_df) + len(test_df))
        self.assertEqual(df.shape[1], train_df.shape[1])
        self.assertEqual(df.shape[1], test_df.shape[1])

        tm.assert_index_equal(df.columns, train_df.columns)
        tm.assert_index_equal(df.columns, test_df.columns)

        df = pdml.ModelFrame(datasets.load_digits())
        df.target_name = 'xxx'

        train_df, test_df = df.model_selection.train_test_split()
        tm.assert_index_equal(df.columns, train_df.columns)
        tm.assert_index_equal(df.columns, test_df.columns)
        self.assertEqual(train_df.target_name, 'xxx')
        self.assertEqual(test_df.target_name, 'xxx')

    def test_train_test_split_abbr(self):

        df = pdml.ModelFrame(datasets.load_digits())
        self.assertIsInstance(df, pdml.ModelFrame)

        train_df, test_df = df.ms.train_test_split()
        tm.assert_index_equal(df.columns, train_df.columns)
        tm.assert_index_equal(df.columns, test_df.columns)

        self.assertEqual(len(df), len(train_df) + len(test_df))
        self.assertEqual(df.shape[1], train_df.shape[1])
        self.assertEqual(df.shape[1], test_df.shape[1])

        tm.assert_index_equal(df.columns, train_df.columns)
        tm.assert_index_equal(df.columns, test_df.columns)

        df = pdml.ModelFrame(datasets.load_digits())
        df.target_name = 'xxx'

        train_df, test_df = df.ms.train_test_split()
        tm.assert_index_equal(df.columns, train_df.columns)
        tm.assert_index_equal(df.columns, test_df.columns)
        self.assertEqual(train_df.target_name, 'xxx')
        self.assertEqual(test_df.target_name, 'xxx')

    def test_train_test_split_keep_index(self):
        df = pdml.ModelFrame({'A': [1, 2, 3, 4, 5, 6, 7, 8],
                              'B': [1, 2, 3, 4, 5, 6, 7, 8]},
                             index='a b c d e f g h'.split(' '))
        tr, te = df.ms.train_test_split(random_state=self.random_state)
        tm.assert_frame_equal(tr, df.loc[['g', 'a', 'e', 'f', 'd', 'h']])
        tm.assert_frame_equal(te, df.loc[['c', 'b']])

        tr, te = df.ms.train_test_split(random_state=self.random_state, reset_index=True)
        tm.assert_frame_equal(tr, df.loc[['g', 'a', 'e', 'f', 'd', 'h']].reset_index(drop=True))
        tm.assert_frame_equal(te, df.loc[['c', 'b']].reset_index(drop=True))

        df = pdml.ModelFrame({'A': [1, 2, 3, 4, 5, 6, 7, 8],
                              'B': [1, 2, 3, 4, 5, 6, 7, 8]},
                             index='a b c d e f g h'.split(' '),
                             target=[1, 2, 3, 4, 5, 6, 7, 8])
        tr, te = df.ms.train_test_split(random_state=self.random_state)
        tm.assert_frame_equal(tr, df.loc[['g', 'a', 'e', 'f', 'd', 'h']])
        tm.assert_numpy_array_equal(tr.target.values, np.array([7, 1, 5, 6, 4, 8]))
        tm.assert_frame_equal(te, df.loc[['c', 'b']])
        tm.assert_numpy_array_equal(te.target.values, np.array([3, 2]))

        tr, te = df.ms.train_test_split(random_state=self.random_state, reset_index=True)
        tm.assert_frame_equal(tr, df.loc[['g', 'a', 'e', 'f', 'd', 'h']].reset_index(drop=True))
        tm.assert_numpy_array_equal(tr.target.values, np.array([7, 1, 5, 6, 4, 8]))
        tm.assert_frame_equal(te, df.loc[['c', 'b']].reset_index(drop=True))
        tm.assert_numpy_array_equal(te.target.values, np.array([3, 2]))

    def test_cross_val_score(self):
        import sklearn.svm as svm
        digits = datasets.load_digits()

        df = pdml.ModelFrame(digits)
        clf = svm.SVC(kernel=str('linear'), C=1)
        result = df.model_selection.cross_val_score(clf, cv=5)
        expected = ms.cross_val_score(clf, X=digits.data, y=digits.target, cv=5)
        self.assert_numpy_array_almost_equal(result, expected)

    def test_permutation_test_score(self):
        import sklearn.svm as svm
        iris = datasets.load_iris()

        df = pdml.ModelFrame(iris)
        clf = svm.SVC(kernel=str('linear'), C=1)
        result = df.model_selection.permutation_test_score(clf, cv=5)
        expected = ms.permutation_test_score(clf, iris.data, y=iris.target, cv=5)

        self.assertEqual(len(result), 3)
        self.assertEqual(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])
        self.assertEqual(result[2], expected[2])

    def test_check_cv(self):
        iris = datasets.load_iris()

        df = pdml.ModelFrame(iris)
        result = df.model_selection.check_cv(cv=5)
        self.assertIsInstance(result, ms.KFold)

    def test_StratifiedShuffleSplit(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)
        sf1 = df.model_selection.StratifiedShuffleSplit(random_state=self.random_state)
        sf2 = ms.StratifiedShuffleSplit(random_state=self.random_state)

        # consume generator
        ind1 = [x for x in sf1.split(df.data.values, df.target.values)]
        ind2 = [x for x in sf2.split(iris.data, iris.target)]

        for i1, i2 in zip(ind1, ind2):
            self.assertIsInstance(i1, tuple)
            self.assertEqual(len(i1), 2)
            self.assertIsInstance(i2, tuple)
            self.assertEqual(len(i2), 2)
            tm.assert_numpy_array_equal(i1[0], i1[0])
            tm.assert_numpy_array_equal(i1[1], i2[1])

        sf1 = df.model_selection.StratifiedShuffleSplit(random_state=self.random_state)
        with tm.assert_produces_warning(FutureWarning):
            gen = df.model_selection.iterate(sf1)

        # StratifiedShuffleSplit is not a subclass of BaseCrossValidator
        for train_df, test_df in gen:
            self.assertIsInstance(train_df, pdml.ModelFrame)
            self.assertIsInstance(test_df, pdml.ModelFrame)
            tm.assert_index_equal(df.columns, train_df.columns)
            tm.assert_index_equal(df.columns, test_df.columns)

            self.assertEqual(len(df), len(train_df) + len(test_df))

    def test_nested_cross_validation(self):
        # http://scikit-learn.org/stable/auto_examples/model_selection/plot_nested_cross_validation_iris.html#sphx-glr-auto-examples-model-selection-plot-nested-cross-validation-iris-py

        # Number of random trials
        NUM_TRIALS = 30

        # Load the dataset
        iris = datasets.load_iris()
        X_iris = iris.data
        y_iris = iris.target

        p_grid = {"C": [1, 10, 100],
                  "gamma": [.01, .1]}

        svr = svm.SVC(kernel="rbf")
        expected = np.zeros(NUM_TRIALS)

        for i in range(NUM_TRIALS):
            inner_cv = ms.KFold(n_splits=4, shuffle=True, random_state=i)
            outer_cv = ms.KFold(n_splits=4, shuffle=True, random_state=i)

            clf = ms.GridSearchCV(estimator=svr, param_grid=p_grid, cv=inner_cv)
            clf.fit(X_iris, y_iris)

            nested_score = ms.cross_val_score(clf, X=X_iris, y=y_iris, cv=outer_cv)
            expected[i] = nested_score.mean()

        df = pdml.ModelFrame(datasets.load_iris())
        svr = df.svm.SVC(kernel="rbf")

        result = np.zeros(NUM_TRIALS)

        for i in range(NUM_TRIALS):
            inner_cv = df.ms.KFold(n_splits=4, shuffle=True, random_state=i)
            outer_cv = df.ms.KFold(n_splits=4, shuffle=True, random_state=i)

            clf = df.ms.GridSearchCV(estimator=svr, param_grid=p_grid, cv=inner_cv)
            df.fit(clf)

            nested_score = df.ms.cross_val_score(clf, cv=outer_cv)
            result[i] = nested_score.mean()

        tm.assert_numpy_array_equal(result, expected)


class TestHyperParameterOptimizer(tm.TestCase):

    def test_grid_search(self):
        tuned_parameters = [{'kernel': ['rbf'], 'gamma': [1e-3, 1e-4],
                             'C': [1, 10, 100]},
                            {'kernel': ['linear'], 'C': [1, 10, 100]}]

        df = pdml.ModelFrame(datasets.load_digits())
        cv = df.model_selection.GridSearchCV(df.svm.SVC(C=1), tuned_parameters, cv=5)

        with tm.RNGContext(1):
            df.fit(cv)

        result = df.model_selection.describe(cv)
        expected = pd.DataFrame(cv.cv_results_)
        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_frame_equal(result, expected)


class TestModelValidation(tm.TestCase):

    def test_learning_curve(self):
        digits = datasets.load_digits()
        df = pdml.ModelFrame(digits)

        result = df.model_selection.learning_curve(df.naive_bayes.GaussianNB())
        expected = ms.learning_curve(nb.GaussianNB(), digits.data, digits.target)

        self.assertEqual(len(result), 3)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])
        self.assert_numpy_array_almost_equal(result[2], expected[2])

    def test_validation_curve(self):
        digits = datasets.load_digits()
        df = pdml.ModelFrame(digits)

        param_range = np.logspace(-2, -1, 2)

        svc = df.svm.SVC(random_state=self.random_state)
        result = df.model_selection.validation_curve(svc, 'gamma',
                                                     param_range)
        expected = ms.validation_curve(svm.SVC(random_state=self.random_state),
                                       digits.data, digits.target,
                                       'gamma', param_range)

        self.assertEqual(len(result), 2)
        self.assert_numpy_array_almost_equal(result[0], expected[0])
        self.assert_numpy_array_almost_equal(result[1], expected[1])



#!/usr/bin/env python

import warnings

import pandas.compat as compat

import pandas_ml.misc as misc
from pandas_ml.compat import Appender


_shared_docs = dict()

_shared_docs['skaccessor'] = """
    Property to access ``sklearn.%(module)s``. See :mod:`pandas_ml.skaccessors.%(module)s`
    """
_shared_docs['skaccessor_nolink'] = """
    Property to access ``sklearn.%(module)s``
    """


class ModelTransformer(object):
    """
    Base class for ``ModelFrame`` and ``ModelFrame``
    """

    def _check_attr(self, estimator, method_name):
        if not hasattr(estimator, method_name):
            msg = "class {0} doesn't have {1} method"
            raise ValueError(msg.format(type(estimator), method_name))
        return getattr(estimator, method_name)

    def _call(self, estimator, method_name, *args, **kwargs):
        # must be overrided
        raise NotImplementedError

    _shared_docs['estimator_methods'] = """
        Call estimator's %(funcname)s method.

        Parameters
        ----------
        args : arguments passed to %(funcname)s method
        kwargs : keyword arguments passed to %(funcname)s method

        Returns
        -------
        %(returned)s
        """

    def _get_method_mapper(self, estimator, method):
        # overridden in ModelFrame
        return None

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='fit', returned='returned : None or fitted estimator'))
    def fit(self, estimator, *args, **kwargs):
        mapped = self._get_method_mapper(estimator, 'fit')
        if mapped is not None:
            result = mapped(self, estimator, *args, **kwargs)
            # save estimator when succeeded
            self.estimator = estimator
            return result
        return self._call(estimator, 'fit', *args, **kwargs)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='transform', returned='returned : transformed result'))
    def transform(self, estimator, *args, **kwargs):
        if isinstance(estimator, compat.string_types):
            # transform using patsy
            return misc.transform_with_patsy(estimator, self, *args, **kwargs)

        # whether to delegate _AccessorMethods.transform
        mapped = self._get_method_mapper(estimator, 'transform')
        if mapped is not None:
            result = mapped(self, estimator, *args, **kwargs)
            # save estimator when succeeded
            self.estimator = estimator
            return result

        transformed = self._call(estimator, 'transform', *args, **kwargs)
        return self._wrap_transform(transformed)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='fit_transform', returned='returned : transformed result'))
    def fit_transform(self, estimator, *args, **kwargs):
        transformed = self._call(estimator, 'fit_transform', *args, **kwargs)
        return self._wrap_transform(transformed)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='inverse_transform', returned='returned : transformed result'))
    def inverse_transform(self, estimator, *args, **kwargs):
        transformed = self._call(estimator, 'inverse_transform', *args, **kwargs)
        transformed = self._wrap_transform(transformed)
        return transformed

    def _wrap_transform(self, transformed):
        raise NotImplementedError


class ModelPredictor(ModelTransformer):
    """
    Base class for ``ModelFrame`` and ``ModelFrameGroupBy``
    """

    @property
    def estimator(self):
        """
        Return most recently used estimator

        Returns
        -------
        estimator : estimator
        """
        if not hasattr(self, '_estimator'):
            self._estimator = None
        return self._estimator

    @estimator.setter
    def estimator(self, value):
        if not hasattr(self, '_estimator') or self._estimator is not value:
            self._estimator = value
            self._predicted = None
            self._proba = None
            self._log_proba = None
            self._decision = None

    @property
    def predicted(self):
        """
        Return current estimator's predicted results

        Returns
        -------
        predicted : ``ModelSeries``
        """
        if not hasattr(self, '_predicted') or self._predicted is None:
            self._predicted = self.predict(self.estimator)
            msg = "Automatically call '{0}.predict()'' to get predicted results"
            warnings.warn(msg.format(self.estimator.__class__.__name__))
        return self._predicted

    @property
    def proba(self):
        """
        Return current estimator's probabilities

        Returns
        -------
        probabilities : ``ModelFrame``
        """
        if not hasattr(self, '_proba') or self._proba is None:
            self._proba = self.predict_proba(self.estimator)
            msg = "Automatically call '{0}.predict_proba()' to get probabilities"
            warnings.warn(msg.format(self.estimator.__class__.__name__))
        return self._proba

    @property
    def log_proba(self):
        """
        Return current estimator's log probabilities

        Returns
        -------
        probabilities : ``ModelFrame``
        """
        if not hasattr(self, '_log_proba') or self._log_proba is None:
            self._log_proba = self.predict_log_proba(self.estimator)
            msg = "Automatically call '{0}.predict_log_proba()' to get log probabilities"
            warnings.warn(msg.format(self.estimator.__class__.__name__))
        return self._log_proba

    @property
    def decision(self):
        """
        Return current estimator's decision function

        Returns
        -------
        decisions : ``ModelFrame``
        """
        if not hasattr(self, '_decision') or self._decision is None:
            self._decision = self.decision_function(self.estimator)
            msg = "Automatically call '{0}.decition_function()' to get decision function"
            warnings.warn(msg.format(self.estimator.__class__.__name__))
        return self._decision

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='predict', returned='returned : predicted result'))
    def predict(self, estimator, *args, **kwargs):
        mapped = self._get_method_mapper(estimator, 'predict')
        if mapped is not None:
            result = mapped(self, estimator, *args, **kwargs)
            # save estimator when succeeded
            self.estimator = estimator
            return result
        predicted = self._call(estimator, 'predict', *args, **kwargs)
        return self._wrap_predicted(predicted, estimator)

    def _wrap_predicted(self, predicted, estimator):
        raise NotImplementedError



#!/usr/bin/env python
import pytest

import datetime
import warnings

import numpy as np
import pandas as pd

import sklearn.datasets as datasets
import sklearn.svm as svm

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestModelFrame(tm.TestCase):

    def test_version(self):
        self.assertTrue(len(pdml.__version__) > 1)

    def test_frame_instance(self):

        df = pdml.ModelFrame(datasets.load_digits())
        self.assertIsInstance(df, pdml.ModelFrame)

        train_df, test_df = df.model_selection.train_test_split()

        self.assertIsInstance(train_df, pdml.ModelFrame)
        self.assertIsInstance(test_df, pdml.ModelFrame)
        self.assertIsInstance(train_df.iloc[:, 2:3], pdml.ModelFrame)

    def test_frame_init_df_series(self):
        # initialization by dataframe and no-named series
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='.target')

        mdf = pdml.ModelFrame(df, target=s)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        s = pd.Series([1, 2, 3])
        with pytest.raises(ValueError, match='data and target must have equal index'):
            mdf = pdml.ModelFrame(df, target=s)

        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='XXX')
        mdf = pdml.ModelFrame(df, target=s)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['XXX', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, 'XXX')
        self.assertEqual(mdf.target_name, 'XXX')

    def test_frame_init_df_str(self):
        # initialization by dataframe and str
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])

        mdf = pdml.ModelFrame(df, target='A')
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df[['B', 'C']])
        tm.assert_series_equal(mdf.target, df['A'])
        self.assertEqual(mdf.target.name, 'A')
        self.assertEqual(mdf.target_name, 'A')

        msg = "Specified target 'X' is not included in data"
        with pytest.raises(ValueError, match=msg):
            mdf = pdml.ModelFrame(df, target='X')

    def test_frame_init_dict_list(self):
        # initialization by dataframe and list
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = [1, 2, 3]
        mdf = pdml.ModelFrame(df, target=s)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        expected = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='.target')
        tm.assert_series_equal(mdf.target, expected)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        df = {'A': [1, 2, 3],
              'B': [4, 5, 6],
              'C': [7, 8, 9]}
        s = [1, 2, 3]
        mdf = pdml.ModelFrame(df, target=s)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index([0, 1, 2]))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))
        expected = pd.DataFrame(df)
        tm.assert_frame_equal(mdf.data, expected)
        expected = pd.Series([1, 2, 3], index=[0, 1, 2], name='.target')
        tm.assert_series_equal(mdf.target, expected)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        mdf = pdml.ModelFrame(df, target='A')
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index([0, 1, 2]))
        tm.assert_index_equal(mdf.columns, pd.Index(['A', 'B', 'C']))
        expected = pd.DataFrame(df)
        tm.assert_frame_equal(mdf.data, expected[['B', 'C']])
        tm.assert_series_equal(mdf.target, expected['A'])
        self.assertEqual(mdf.target.name, 'A')
        self.assertEqual(mdf.target_name, 'A')

        mdf = pdml.ModelFrame({'A': [1, 2, 3],
                               'B': [4, 5, 6],
                               'C': [7, 8, 9]},
                              index=['a', 'b', 'c'],
                              columns=['A', 'B', 'C'])
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, mdf)
        self.assertEqual(mdf.target_name, '.target')

    def test_frame_init_df_array_series(self):
        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='.target')
        mdf = pdml.ModelFrame(np.array([[1, 2, 3], [4, 5, 6],
                                        [7, 8, 9]]), target=s,
                              index=['a', 'b', 'c'], columns=['A', 'B', 'C'])

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))

        expected = pd.DataFrame(np.array([[1, 2, 3], [4, 5, 6],
                                          [7, 8, 9]]),
                                index=['a', 'b', 'c'], columns=['A', 'B', 'C'])
        tm.assert_frame_equal(mdf.data, expected)
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

    def test_frame_init_dict_list_series_index(self):
        # initialization by dataframe and list
        df = {'A': [1, 2, 3], 'B': [4, 5, 6], 'C': [7, 8, 9]}
        target = pd.Series([9, 8, 7], name='X', index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=target)

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['X', 'A', 'B', 'C']))
        expected = pd.DataFrame(df, index=['a', 'b', 'c'])
        tm.assert_frame_equal(mdf.data, expected)
        tm.assert_series_equal(mdf.target, target)
        self.assertEqual(mdf.target.name, 'X')
        self.assertEqual(mdf.target_name, 'X')

    def test_frame_init_df_none(self):
        # initialization by dataframe and none
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])

        mdf = pdml.ModelFrame(df, target=None)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        self.assertTrue(mdf.has_data())
        self.assertTrue(mdf.target is None)
        self.assertEqual(mdf.target_name, '.target')

    def test_frame_init_df_duplicated(self):
        # initialization by dataframe and duplicated target
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          columns=['A', 'B', 'C'])
        s = pd.Series([10, 11, 12], name='A')

        msg = "data and target must have unique names"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(df, target=s)

        df = pdml.ModelFrame({'A': [1, 2, 3],
                              'B': [4, 5, 6],
                              'C': [7, 8, 9]},
                             columns=['A', 'B', 'C'])
        df.target = pd.Series([10, 11, 12], name='A')
        expected = pdml.ModelFrame({'A': [10, 11, 12],
                                    'B': [4, 5, 6],
                                    'C': [7, 8, 9]},
                                   columns=['A', 'B', 'C'])
        tm.assert_frame_equal(df, expected)

    def test_frame_data_none(self):
        msg = "ModelFrame must have either data or target"
        with pytest.raises(ValueError, match=msg):
            mdf = pdml.ModelFrame(None)

        msg = "target must be list-like when data is None"
        with pytest.raises(ValueError, match=msg):
            mdf = pdml.ModelFrame(None, target='X')

        # initialization without data
        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='.target')
        mdf = pdml.ModelFrame(None, target=s)

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 1))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target']))
        self.assertTrue(mdf.data is None)
        self.assertFalse(mdf.has_data())
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

    def test_frame_slice(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        expected = ['.target'] + iris.feature_names
        self.assertEqual(df.columns.tolist(), expected)

        s = df['.target']
        self.assertIsInstance(s, pdml.ModelSeries)
        s = s[1:5]
        self.assertIsInstance(s, pdml.ModelSeries)

        s = df[['.target']]
        self.assertIsInstance(s, pdml.ModelFrame)

    def test_frame_data_proparty(self):
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='.target')

        mdf = pdml.ModelFrame(df, target=s)
        self.assertIsInstance(mdf, pdml.ModelFrame)

        new = pd.DataFrame({'X': [1, 2, 3],
                            'Y': [4, 5, 6]},
                           index=['a', 'b', 'c'],
                           columns=['X', 'Y'])
        # set data property
        mdf.data = new

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'X', 'Y']))
        tm.assert_frame_equal(mdf.data, new)
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        new = pdml.ModelFrame({'M': [1, 2, 3],
                               'N': [4, 5, 6]},
                              index=['a', 'b', 'c'],
                              columns=['M', 'N'])

        # set data property
        mdf.data = new

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'M', 'N']))
        tm.assert_frame_equal(mdf.data, new)
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        new = pd.DataFrame({'.target': [1, 2, 3],
                            'K': [4, 5, 6]},
                           index=['a', 'b', 'c'])

        # unable to set data if passed value has the same column as the target
        msg = "Passed data has the same column name as the target '.target'"
        with pytest.raises(ValueError, match=msg):
            mdf.data = new

        # unable to set ModelFrame with target attribute
        msg = "Cannot update with ModelFrame which has target attribute"
        with pytest.raises(ValueError, match=msg):
            mdf.data = mdf

        # set delete property
        del mdf.data
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 1))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target']))
        self.assertTrue(mdf.data is None)
        tm.assert_series_equal(mdf.target, s)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

    def test_frame_data_proparty_series(self):
        df = pdml.ModelFrame({'A': [1, 2, 3],
                              'B': [4, 5, 6]},
                             target=[7, 8, 9],
                             index=['a', 'b', 'c'])
        df.data = df['A']
        exp = pdml.ModelFrame({'A': [1, 2, 3]},
                              target=[7, 8, 9],
                              index=['a', 'b', 'c'])
        tm.assert_frame_equal(df, exp)

        df = pdml.ModelFrame({'A': [1, 2, 3],
                              'B': [4, 5, 6]},
                             target=[7, 8, 9],
                             index=['a', 'b', 'c'])
        df.data = pd.Series([1, 2, 3], name='x', index=['a', 'b', 'c'])
        exp = pdml.ModelFrame({'x': [1, 2, 3]},
                              target=[7, 8, 9],
                              index=['a', 'b', 'c'])
        tm.assert_frame_equal(df, exp)

        df = pdml.ModelFrame({'A': [1, 2, 3],
                              'B': [4, 5, 6]},
                             target=[7, 8, 9],
                             index=['a', 'b', 'c'])
        with pytest.raises(TypeError):
            df.data = [1, 2, 3]

    def test_frame_target_proparty(self):
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=s)

        new = pd.Series([4, 5, 6], index=['a', 'b', 'c'], name='.target')
        # set target property
        mdf.target = new

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        tm.assert_series_equal(mdf.target, new)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        with tm.assert_produces_warning(UserWarning):
            new = pd.Series([4, 5, 6], index=['a', 'b', 'c'], name='xxx')
            # set target property
            mdf.target = new

            self.assertIsInstance(mdf, pdml.ModelFrame)
            self.assertEqual(mdf.shape, (3, 4))
            tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
            tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))
            tm.assert_frame_equal(mdf.data, df)

            exp_target = pd.Series(new, name='.target')
            tm.assert_series_equal(mdf.target, exp_target)
            self.assertEqual(mdf.target.name, '.target')
            self.assertEqual(mdf.target_name, '.target')

        new = pd.Series([4, 5, 6], name='.target')
        with pytest.raises(ValueError, match='data and target must have equal index'):
            mdf.target = new

        # set target property
        mdf.target = [7, 8, 9]

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['.target', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        expected = pd.Series([7, 8, 9], index=['a', 'b', 'c'], name='.target')
        tm.assert_series_equal(mdf.target, expected)
        self.assertEqual(mdf.target.name, '.target')
        self.assertEqual(mdf.target_name, '.target')

        if pdml.compat._PANDAS_ge_023:
            msg = 'Length of passed values is 2, index implies 3'
        else:
            msg = 'Wrong number of items passed 2, placement implies 3'

        with pytest.raises(ValueError, match=msg):
            mdf.target = [1, 2]

        # set target property
        mdf.target = None

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        self.assertEqual(mdf.target_name, '.target')

    def test_frame_delete_proparty(self):
        mdf = pdml.ModelFrame(None, target=[1, 2, 3])
        msg = 'ModelFrame must have either data or target'

        with pytest.raises(ValueError, match=msg):
            del mdf.target

        with pytest.raises(ValueError, match=msg):
            mdf.target = None

        mdf = pdml.ModelFrame([1, 2, 3])
        msg = 'ModelFrame must have either data or target'

        with pytest.raises(ValueError, match=msg):
            del mdf.data

        with pytest.raises(ValueError, match=msg):
            mdf.data = None

    def test_frame_target_object(self):
        df = pd.DataFrame({datetime.datetime(2014, 1, 1): [1, 2, 3],
                           datetime.datetime(2015, 1, 1): [4, 5, 6],
                           datetime.datetime(2016, 1, 1): [7, 8, 9]},
                          index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=datetime.datetime(2016, 1, 1))

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 3))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        expected = pd.DatetimeIndex(['2014-01-01', '2015-01-01', '2016-01-01'])
        tm.assert_index_equal(mdf.columns, expected)
        tm.assert_frame_equal(mdf.data, df.iloc[:, :2])
        expected = pd.Series([7, 8, 9], index=['a', 'b', 'c'],
                             name=pd.Timestamp('2016-01-01'))
        tm.assert_series_equal(mdf.target, expected)
        self.assertEqual(mdf.target.name, datetime.datetime(2016, 1, 1))
        self.assertEqual(mdf.target_name, datetime.datetime(2016, 1, 1))

    def test_frame_target_object_set(self):

        df = pd.DataFrame({datetime.datetime(2014, 1, 1): [1, 2, 3],
                           datetime.datetime(2015, 1, 1): [4, 5, 6],
                           datetime.datetime(2016, 1, 1): [7, 8, 9]},
                          index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df)

        mdf.target = pd.Series(['A', 'B', 'C'], index=['a', 'b', 'c'], name=5)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        expected = pd.Index([5, datetime.datetime(2014, 1, 1),
                             datetime.datetime(2015, 1, 1), datetime.datetime(2016, 1, 1)])
        tm.assert_index_equal(mdf.columns, expected)
        tm.assert_frame_equal(mdf.data, df)
        expected = pd.Series(['A', 'B', 'C'], index=['a', 'b', 'c'], name=5)
        tm.assert_series_equal(mdf.target, expected)
        self.assertEqual(mdf.target.name, 5)

        # name will be ignored if ModelFrame already has a target
        mdf.target = pd.Series([10, 11, 12], index=['a', 'b', 'c'], name='X')
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        expected = pd.Index([5,
                             datetime.datetime(2014, 1, 1),
                             datetime.datetime(2015, 1, 1),
                             datetime.datetime(2016, 1, 1)])
        tm.assert_index_equal(mdf.columns, expected)
        tm.assert_frame_equal(mdf.data, df)
        expected = pd.Series([10, 11, 12], index=['a', 'b', 'c'], name=5)
        tm.assert_series_equal(mdf.target, expected)
        self.assertEqual(mdf.target.name, 5)

    def test_predict_proba(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        models = ['SVC']
        for model in models:
            mod1 = getattr(df.svm, model)(probability=True,
                                          random_state=self.random_state)
            mod2 = getattr(svm, model)(probability=True,
                                       random_state=self.random_state)

            df.fit(mod1)
            mod2.fit(iris.data, iris.target)

            result = df.predict(mod1)
            expected = mod2.predict(iris.data)

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

            result = df.predict_proba(mod1)
            expected = mod2.predict_proba(iris.data)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_index_equal(result.index, df.index)
            self.assert_numpy_array_almost_equal(result.values, expected)
            self.assert_numpy_array_almost_equal(df.proba.values, expected)

            result = df.predict_log_proba(mod1)
            expected = mod2.predict_log_proba(iris.data)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_index_equal(result.index, df.index)
            self.assert_numpy_array_almost_equal(result.values, expected)
            self.assert_numpy_array_almost_equal(df.log_proba.values, expected)

            result = df.decision_function(mod1)
            expected = mod2.decision_function(iris.data)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_index_equal(result.index, df.index)
            self.assert_numpy_array_almost_equal(result.values, expected)
            self.assert_numpy_array_almost_equal(df.decision.values, expected)

            # not reset if estimator is identical
            df.fit(mod1)
            self.assertFalse(df._predicted is None)
            self.assertFalse(df._proba is None)
            self.assertFalse(df._log_proba is None)
            self.assertFalse(df._decision is None)

            # reset estimator
            mod3 = getattr(df.svm, model)(probability=True,
                                          random_state=self.random_state)
            df.fit(mod3)
            self.assertTrue(df._predicted is None)
            self.assertTrue(df._proba is None)
            self.assertTrue(df._log_proba is None)
            self.assertTrue(df._decision is None)

    def test_predict_automatic(self):
        with warnings.catch_warnings():
            warnings.simplefilter("always", UserWarning)

            iris = datasets.load_iris()
            df = pdml.ModelFrame(iris)

            model = 'SVC'

            df = pdml.ModelFrame(iris)
            mod1 = getattr(df.svm, model)(probability=True,
                                          random_state=self.random_state)
            mod2 = getattr(svm, model)(probability=True,
                                       random_state=self.random_state)

            df.fit(mod1)
            mod2.fit(iris.data, iris.target)

            # test automatically calls related methods
            with tm.assert_produces_warning(UserWarning):
                result = df.predicted
            expected = mod2.predict(iris.data)

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

            # with tm.assert_produces_warning(UserWarning):
            result = df.proba
            expected = mod2.predict_proba(iris.data)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_index_equal(result.index, df.index)
            self.assert_numpy_array_almost_equal(result.values, expected)

            with tm.assert_produces_warning(UserWarning):
                result = df.log_proba
            expected = mod2.predict_log_proba(iris.data)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_index_equal(result.index, df.index)
            self.assert_numpy_array_almost_equal(result.values, expected)

            # with tm.assert_produces_warning(UserWarning):
            result = df.decision
            expected = mod2.decision_function(iris.data)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_index_equal(result.index, df.index)
            self.assert_numpy_array_almost_equal(result.values, expected)

        warnings.simplefilter("default")

    def test_frame_metadata(self):
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = pd.Series([1, 2, 3], index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=s)

        self.assertEqual(mdf.target_name, '.target')
        sliced = mdf.iloc[1:2, ]
        self.assertIsInstance(sliced, pdml.ModelFrame)
        self.assertTrue(sliced.has_target())
        self.assertEqual(sliced.target_name, '.target')

        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        mdf = pdml.ModelFrame(df, target='A')

        self.assertEqual(mdf.target_name, 'A')
        sliced = mdf.loc[['a', 'b'], :]
        self.assertIsInstance(sliced, pdml.ModelFrame)
        self.assertTrue(sliced.has_target())
        self.assertEqual(sliced.target_name, 'A')

        sliced = mdf.iloc[[1, 2], :]
        self.assertIsInstance(sliced, pdml.ModelFrame)
        self.assertTrue(sliced.has_target())
        self.assertEqual(sliced.target_name, 'A')


class TestModelFrameMultiTarges(tm.TestCase):

    def test_frame_instance(self):

        df = pdml.ModelFrame(datasets.load_digits())
        self.assertIsInstance(df, pdml.ModelFrame)

        train_df, test_df = df.model_selection.train_test_split()

        self.assertIsInstance(train_df, pdml.ModelFrame)
        self.assertIsInstance(test_df, pdml.ModelFrame)
        self.assertIsInstance(train_df.iloc[:, 2:3], pdml.ModelFrame)

    def test_frame_init_df_df(self):
        # initialization by dataframe and dataframe
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        target = pd.DataFrame({'t1': [10, 11, 12],
                               't2': [13, 14, 15]},
                              index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=target)

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 5))
        expected = pd.DataFrame({'t1': [10, 11, 12],
                                 't2': [13, 14, 15],
                                 'A': [1, 2, 3],
                                 'B': [4, 5, 6],
                                 'C': [7, 8, 9]},
                                index=['a', 'b', 'c'],
                                columns=['t1', 't2', 'A', 'B', 'C'])
        tm.assert_frame_equal(mdf, expected)
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns,
                              pd.Index(['t1', 't2', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        tm.assert_frame_equal(mdf.target, target)
        tm.assert_index_equal(mdf.target.columns, pd.Index(['t1', 't2']))
        tm.assert_index_equal(mdf.target_name, pd.Index(['t1', 't2']))
        self.assertTrue(mdf.has_multi_targets())

        target = pd.DataFrame({'t1': [10, 11, 12], 't2': [13, 14, 15]})
        msg = 'data and target must have equal index'
        with pytest.raises(ValueError, match=msg):
            mdf = pdml.ModelFrame(df, target=target)

        # single column DataFrame will results in single target column
        target = pd.DataFrame({'t1': [10, 11, 12]}, index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=target)
        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 4))
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns, pd.Index(['t1', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)

        target = pd.Series([10, 11, 12], name='t1', index=['a', 'b', 'c'])
        tm.assert_series_equal(mdf.target, target)
        self.assertEqual(mdf.target_name, 't1')

    def test_frame_init_df_target_setter(self):
        # initialization by dataframe and dataframe
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        mdf = pdml.ModelFrame(df)
        self.assertFalse(mdf.has_target())
        target = pd.DataFrame({'t1': [10, 11, 12],
                               't2': [13, 14, 15]},
                              index=['a', 'b', 'c'])
        mdf.target = target

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 5))
        expected = pd.DataFrame({'t1': [10, 11, 12],
                                 't2': [13, 14, 15],
                                 'A': [1, 2, 3],
                                 'B': [4, 5, 6],
                                 'C': [7, 8, 9]},
                                index=['a', 'b', 'c'],
                                columns=['t1', 't2', 'A', 'B', 'C'])
        tm.assert_frame_equal(mdf, expected)
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns,
                              pd.Index(['t1', 't2', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        tm.assert_frame_equal(mdf.target, target)
        tm.assert_index_equal(mdf.target.columns, pd.Index(['t1', 't2']))
        tm.assert_index_equal(mdf.target_name, pd.Index(['t1', 't2']))
        self.assertTrue(mdf.has_multi_targets())

        target = pd.DataFrame({'x1': [20, 21, 22],
                               'x2': [23, 24, 25]},
                              index=['a', 'b', 'c'])

        with tm.assert_produces_warning(UserWarning):
            # when the target has the same length as the target_name,
            # is renamed to existing target ['t1', 't2']
            mdf.target = target

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 5))
        expected = pd.DataFrame({'t1': [20, 21, 22],
                                 't2': [23, 24, 25],
                                 'A': [1, 2, 3],
                                 'B': [4, 5, 6],
                                 'C': [7, 8, 9]},
                                index=['a', 'b', 'c'],
                                columns=['t1', 't2', 'A', 'B', 'C'])
        tm.assert_frame_equal(mdf, expected)
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns,
                              pd.Index(['t1', 't2', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        expected = pd.DataFrame({'t1': [20, 21, 22],
                                 't2': [23, 24, 25]},
                                index=['a', 'b', 'c'])
        tm.assert_frame_equal(mdf.target, expected)
        tm.assert_index_equal(mdf.target.columns, pd.Index(['t1', 't2']))
        tm.assert_index_equal(mdf.target_name, pd.Index(['t1', 't2']))
        self.assertTrue(mdf.has_multi_targets())

        target = pd.DataFrame({'x1': [20, 21, 22],
                               'x2': [23, 24, 25],
                               'x3': [25, 26, 27]},
                              index=['a', 'b', 'c'])

        # when the target has the different length as the target_name,
        # target is being replaced
        mdf.target = target

        self.assertIsInstance(mdf, pdml.ModelFrame)
        self.assertEqual(mdf.shape, (3, 6))
        expected = pd.DataFrame({'x1': [20, 21, 22],
                                 'x2': [23, 24, 25],
                                 'x3': [25, 26, 27],
                                 'A': [1, 2, 3],
                                 'B': [4, 5, 6],
                                 'C': [7, 8, 9]},
                                index=['a', 'b', 'c'],
                                columns=['x1', 'x2', 'x3', 'A', 'B', 'C'])
        tm.assert_frame_equal(mdf, expected)
        tm.assert_index_equal(mdf.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(mdf.columns,
                              pd.Index(['x1', 'x2', 'x3', 'A', 'B', 'C']))
        tm.assert_frame_equal(mdf.data, df)
        tm.assert_frame_equal(mdf.target, target)
        tm.assert_index_equal(mdf.target.columns,
                              pd.Index(['x1', 'x2', 'x3']))
        tm.assert_index_equal(mdf.target_name, pd.Index(['x1', 'x2', 'x3']))
        self.assertTrue(mdf.has_multi_targets())

    def test_frame_init_df_duplicated_columns(self):
        # initialization by dataframe and dataframe which have same columns
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]})
        target = pd.DataFrame({'A': [10, 11, 12],
                               'B': [13, 14, 15]})

        mdf = pdml.ModelFrame(df, target=target)

        cols = pd.MultiIndex.from_tuples([('.target', 'A'),
                                          ('.target', 'B'),
                                          ('.data', 'A'),
                                          ('.data', 'B'),
                                          ('.data', 'C')])
        expected = pd.DataFrame({('.target', 'A'): [10, 11, 12],
                                 ('.target', 'B'): [13, 14, 15],
                                 ('.data', 'A'): [1, 2, 3],
                                 ('.data', 'B'): [4, 5, 6],
                                 ('.data', 'C'): [7, 8, 9]},
                                columns=cols)
        tm.assert_frame_equal(mdf, expected)



#!/usr/bin/env python

try:
    import sklearn.multioutput as multioutput
except ImportError:
    pass

import numpy as np
import pandas as pd

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestMultiOutput(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.multioutput.MultiOutputRegressor,
                      multioutput.MultiOutputRegressor)
        self.assertIs(df.multioutput.MultiOutputClassifier,
                      multioutput.MultiOutputClassifier)

    def test_multioutput(self):

        # http://scikit-learn.org/stable/auto_examples/ensemble/plot_random_forest_regression_multioutput.html#sphx-glr-auto-examples-ensemble-plot-random-forest-regression-multioutput-py

        from sklearn.multioutput import MultiOutputRegressor
        from sklearn.ensemble import RandomForestRegressor

        # Create a random dataset
        rng = np.random.RandomState(1)
        X = np.sort(200 * rng.rand(600, 1) - 100, axis=0)
        y = np.array([np.pi * np.sin(X).ravel(), np.pi * np.cos(X).ravel()]).T
        y += (0.5 - rng.rand(*y.shape))

        df = pdml.ModelFrame(X, target=y)

        max_depth = 30

        rf1 = df.ensemble.RandomForestRegressor(max_depth=max_depth,
                                                random_state=self.random_state)
        reg1 = df.multioutput.MultiOutputRegressor(rf1)

        rf2 = RandomForestRegressor(max_depth=max_depth,
                                    random_state=self.random_state)
        reg2 = MultiOutputRegressor(rf2)

        df.fit(reg1)
        reg2.fit(X, y)

        result = df.predict(reg2)
        expected = pd.DataFrame(reg2.predict(X))
        tm.assert_frame_equal(result, expected)



#!/usr/bin/env python

import pandas as pd
import pandas.compat as compat

from pandas_ml.compat import Appender
from pandas_ml.core.base import _BaseEstimator
from pandas_ml.core.frame import ModelFrame
from pandas_ml.core.generic import ModelPredictor, _shared_docs
from pandas_ml.core.series import ModelSeries


@Appender(pd.core.groupby.GroupBy.__doc__)
def groupby(obj, by, **kwds):
    if isinstance(obj, ModelSeries):
        klass = ModelSeriesGroupBy
    elif isinstance(obj, ModelFrame):
        klass = ModelFrameGroupBy
    else:  # pragma: no cover
        raise TypeError('invalid type: %s' % type(obj))

    return klass(obj, by, **kwds)


class ModelSeriesGroupBy(pd.core.groupby.SeriesGroupBy):
    pass


class ModelFrameGroupBy(pd.core.groupby.DataFrameGroupBy, ModelPredictor):

    _internal_caches = ['_estimator', '_predicted', '_proba', '_log_proba', '_decision']
    _internal_names = pd.core.groupby.DataFrameGroupBy._internal_names + _internal_caches
    _internal_names_set = set(_internal_names)

    @Appender(_shared_docs['estimator_methods'] %
              dict(funcname='transform', returned='returned : transformed result'))
    def transform(self, func, *args, **kwargs):
        if isinstance(func, GroupedEstimator):
            return ModelPredictor.transform(self, func, *args, **kwargs)
        else:
            return pd.core.groupby.DataFrameGroupBy.transform(self, func, *args, **kwargs)

    def _get_mapper(self, estimator, method_name):
        # mappings are handled by ModelFrame._get_mapper
        return None

    def _call(self, estimator, method_name, *args, **kwargs):
        if method_name in ['fit', 'fit_transform']:
            estimator = GroupedEstimator(estimator, self)

        if not isinstance(estimator, GroupedEstimator):
            raise ValueError('Class {0} is not GroupedEstimator'.format(estimator.__class__.__name__))

        results = {}
        for name, group in self:
            e = estimator.groups[name]
            method = getattr(group, method_name)
            results[name] = method(e)
        self.estimator = estimator
        if method_name == 'fit':
            return estimator
        else:
            return results

    def _wrap_transform(self, transformed):
        return self._wrap_results(transformed)

    def _wrap_predicted(self, predicted, estimator):
        return self._wrap_results(predicted)

    def _wrap_results(self, results):
        keys = []
        values = []
        for key, value in compat.iteritems(results):
            keys.extend([key] * len(value))
            values.append(value)
        results = pd.concat(values, axis=0, ignore_index=False)
        if isinstance(results, pd.Series):
            results = ModelSeries(results)
            # keys must be list
            results = results.groupby(by=keys)
        elif isinstance(results, pd.DataFrame):
            results = ModelFrame(results)
            # keys must be Series
            results = results.groupby(by=pd.Series(keys))
        else:
            raise ValueError('Unknown type: {0}'.format(results.__class__.__name__))

        return results


class GroupedEstimator(_BaseEstimator):
    """
    Create grouped estimators based on passed estimator
    """
    def __init__(self, estimator, grouped):
        if not isinstance(grouped, pd.core.groupby.DataFrameGroupBy):
            raise ValueError("'grouped' must be DataFrameGroupBy instance")

        import sklearn.base as base

        self.groups = {}

        for name, group in grouped:
            e = base.clone(estimator)
            e = e.set_params(**estimator.get_params())
            self.groups[name] = e



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.multiclass as multiclass
import sklearn.svm as svm

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestMultiClass(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.multiclass.OneVsRestClassifier, multiclass.OneVsRestClassifier)
        self.assertIs(df.multiclass.OneVsOneClassifier, multiclass.OneVsOneClassifier)
        self.assertIs(df.multiclass.OutputCodeClassifier, multiclass.OutputCodeClassifier)

    @pytest.mark.parametrize("algo", ['OneVsOneClassifier',
                                      'OneVsOneClassifier'])
    def test_Classifications(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        svm1 = df.svm.LinearSVC(random_state=self.random_state)
        svm2 = svm.LinearSVC(random_state=self.random_state)
        mod1 = getattr(df.multiclass, algo)(svm1)
        mod2 = getattr(multiclass, algo)(svm2)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    @pytest.mark.parametrize("algo", ['OutputCodeClassifier'])
    def test_Classifications_Random(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        svm1 = df.svm.LinearSVC(random_state=self.random_state)
        svm2 = svm.LinearSVC(random_state=self.random_state)
        mod1 = getattr(df.multiclass, algo)(svm1, random_state=self.random_state)
        mod2 = getattr(multiclass, algo)(svm2, random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.predict(mod1)
        expected = mod2.predict(iris.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)



#!/usr/bin/env python

import numpy as np
import pandas.util.testing as tm

from pandas_ml.compat import plotting
from pandas.util.testing import (assert_produces_warning,           # noqa
                                 close, RNGContext,                 # noqa
                                 assert_index_equal,                # noqa
                                 assert_series_equal,               # noqa
                                 assert_frame_equal,                # noqa
                                 assert_numpy_array_equal)          # noqa


try:
    _flatten = plotting._flatten
except AttributeError:
    import pandas.plotting._tools
    _flatten = pandas.plotting._tools._flatten


class TestCase(object):

    @property
    def random_state(self):
        return np.random.RandomState(1234)

    def format(self, val):
        return '{} (type: {})'.format(val, type(val))

    def format_values(self, left, right):
        fmt = """Input vaues are different:
Left: {}
Right: {}
"""
        return fmt.format(self.format(left), self.format(right))

    def assertEqual(self, left, right):
        assert left == right, self.format_values(left, right)

    def assertIs(self, left, right):
        assert left is right, self.format_values(left, right)

    def assertAlmostEqual(self, left, right):
        assert tm.assert_almost_equal(left, right), self.format_values(left, right)

    def assertIsNone(self, left):
        assert left is None, self.format(left)

    def assertTrue(self, left):
        assert left is True or left is np.bool_(True), self.format(left)

    def assertFalse(self, left):
        assert left is False or left is np.bool_(False), self.format(left)

    def assertIsInstance(self, instance, klass):
        assert isinstance(instance, klass), self.format(instance)

    def assert_numpy_array_almost_equal(self, a, b):
        return np.testing.assert_array_almost_equal(a, b)


class PlottingTestCase(TestCase):

    def teardown_method(self):
        tm.close()

    def _check_axes_shape(self, axes, axes_num=None, layout=None, figsize=(8.0, 6.0)):
        """
        Check expected number of axes is drawn in expected layout

        Parameters
        ----------
        axes : matplotlib Axes object, or its list-like
        axes_num : number
            expected number of axes. Unnecessary axes should be set to invisible.
        layout :  tuple
            expected layout, (expected number of rows , columns)
        figsize : tuple
            expected figsize. default is matplotlib default
        """

        # derived from pandas.tests.test_graphics
        visible_axes = self._flatten_visible(axes)

        if axes_num is not None:
            self.assertEqual(len(visible_axes), axes_num)
            for ax in visible_axes:
                # check something drawn on visible axes
                self.assertTrue(len(ax.get_children()) > 0)

        if layout is not None:
            result = self._get_axes_layout(_flatten(axes))
            self.assertEqual(result, layout)

        if figsize is not None:
            self.assert_numpy_array_equal(np.round(visible_axes[0].figure.get_size_inches()),
                                          np.array(figsize))

    def _get_axes_layout(self, axes):
        x_set = set()
        y_set = set()
        for ax in axes:
            # check axes coordinates to estimate layout
            points = ax.get_position().get_points()
            x_set.add(points[0][0])
            y_set.add(points[0][1])
        return (len(y_set), len(x_set))

    def _flatten_visible(self, axes):
        """
        Flatten axes, and filter only visible

        Parameters
        ----------
        axes : matplotlib Axes object, or its list-like

        """
        axes = _flatten(axes)
        axes = [ax for ax in axes if ax.get_visible()]
        return axes



#!/usr/bin/env python

import pandas as pd

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestModelFrame(tm.TestCase):

    def test_patsy_matrices(self):
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = pd.Series([10, 11, 12], index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=s)

        result = mdf.transform('A ~ B + C')
        self.assertIsInstance(result, pdml.ModelFrame)
        self.assertEqual(result.shape, (3, 4))
        tm.assert_index_equal(result.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(result.columns, pd.Index(['A', 'Intercept', 'B', 'C']))
        expected = pd.DataFrame({'A': [1, 2, 3],
                                 'Intercept': [1, 1, 1],
                                 'B': [4, 5, 6],
                                 'C': [7, 8, 9]},
                                index=['a', 'b', 'c'],
                                columns=['A', 'Intercept', 'B', 'C'],
                                dtype=float)
        tm.assert_frame_equal(result, expected)
        expected = pd.Series([1, 2, 3], index=['a', 'b', 'c'], name='A', dtype=float)
        tm.assert_series_equal(result.target, expected)
        self.assertEqual(result.target.name, 'A')
        self.assertEqual(result.target_name, 'A')

    def test_patsy_matrix(self):
        df = pd.DataFrame({'A': [1, 2, 3],
                           'B': [4, 5, 6],
                           'C': [7, 8, 9]},
                          index=['a', 'b', 'c'],
                          columns=['A', 'B', 'C'])
        s = pd.Series([10, 11, 12], index=['a', 'b', 'c'])
        mdf = pdml.ModelFrame(df, target=s)

        result = mdf.transform('B + C')
        self.assertIsInstance(result, pdml.ModelFrame)
        self.assertEqual(result.shape, (3, 3))
        tm.assert_index_equal(result.index, pd.Index(['a', 'b', 'c']))
        tm.assert_index_equal(result.columns, pd.Index(['Intercept', 'B', 'C']))
        expected = pd.DataFrame({'Intercept': [1, 1, 1],
                                 'B': [4, 5, 6],
                                 'C': [7, 8, 9]},
                                index=['a', 'b', 'c'],
                                columns=['Intercept', 'B', 'C'],
                                dtype=float)
        tm.assert_frame_equal(result, expected)
        self.assertFalse(result.has_target())
        self.assertEqual(result.target_name, '.target')

    def test_patsy_deviation_coding(self):
        df = pdml.ModelFrame({'X': [1, 2, 3, 4, 5], 'Y': [1, 3, 2, 2, 1],
                              'Z': [1, 1, 1, 2, 2]}, target='Z',
                             index=['a', 'b', 'c', 'd', 'e'])

        result = df.transform('C(X, Sum)')
        expected = pd.DataFrame({'Intercept': [1, 1, 1, 1, 1],
                                 'C(X, Sum)[S.1]': [1, 0, 0, 0, -1],
                                 'C(X, Sum)[S.2]': [0, 1, 0, 0, -1],
                                 'C(X, Sum)[S.3]': [0, 0, 1, 0, -1],
                                 'C(X, Sum)[S.4]': [0, 0, 0, 1, -1]},
                                index=['a', 'b', 'c', 'd', 'e'],
                                columns=['Intercept', 'C(X, Sum)[S.1]', 'C(X, Sum)[S.2]',
                                         'C(X, Sum)[S.3]', 'C(X, Sum)[S.4]'],
                                dtype=float)
        tm.assert_frame_equal(result, expected)

        result = df.transform('C(Y, Sum)')
        expected = pd.DataFrame({'Intercept': [1, 1, 1, 1, 1],
                                 'C(Y, Sum)[S.1]': [1, -1, 0, 0, 1],
                                 'C(Y, Sum)[S.2]': [0, -1, 1, 1, 0]},
                                index=['a', 'b', 'c', 'd', 'e'],
                                columns=['Intercept', 'C(Y, Sum)[S.1]', 'C(Y, Sum)[S.2]'],
                                dtype=float)
        tm.assert_frame_equal(result, expected)



#!/usr/bin/env python
import pytest

import numpy as np

import pandas_ml as pdml
import pandas_ml.util.testing as tm

imblean = pytest.importorskip("imblearn")


class TestImbalance(tm.TestCase):

    def test_objectmapper_undersampling(self):
        import imblearn.under_sampling as us
        df = pdml.ModelFrame([])
        self.assertIs(df.imbalance.under_sampling.ClusterCentroids,
                      us.ClusterCentroids)
        self.assertIs(df.imbalance.under_sampling.CondensedNearestNeighbour,
                      us.CondensedNearestNeighbour)
        self.assertIs(df.imbalance.under_sampling.EditedNearestNeighbours,
                      us.EditedNearestNeighbours)
        self.assertIs(df.imbalance.under_sampling.RepeatedEditedNearestNeighbours,
                      us.RepeatedEditedNearestNeighbours)
        self.assertIs(df.imbalance.under_sampling.InstanceHardnessThreshold,
                      us.InstanceHardnessThreshold)
        self.assertIs(df.imbalance.under_sampling.NearMiss,
                      us.NearMiss)
        self.assertIs(df.imbalance.under_sampling.NeighbourhoodCleaningRule,
                      us.NeighbourhoodCleaningRule)
        self.assertIs(df.imbalance.under_sampling.OneSidedSelection,
                      us.OneSidedSelection)
        self.assertIs(df.imbalance.under_sampling.RandomUnderSampler,
                      us.RandomUnderSampler)
        self.assertIs(df.imbalance.under_sampling.TomekLinks,
                      us.TomekLinks)

    def test_objectmapper_oversampling(self):
        import imblearn.over_sampling as os
        df = pdml.ModelFrame([])
        self.assertIs(df.imbalance.over_sampling.ADASYN,
                      os.ADASYN)
        self.assertIs(df.imbalance.over_sampling.RandomOverSampler,
                      os.RandomOverSampler)
        self.assertIs(df.imbalance.over_sampling.SMOTE,
                      os.SMOTE)

    def test_objectmapper_combine(self):
        import imblearn.combine as combine
        df = pdml.ModelFrame([])
        self.assertIs(df.imbalance.combine.SMOTEENN,
                      combine.SMOTEENN)
        self.assertIs(df.imbalance.combine.SMOTETomek,
                      combine.SMOTETomek)

    def test_objectmapper_ensemble(self):
        import imblearn.ensemble as ensemble
        df = pdml.ModelFrame([])
        self.assertIs(df.imbalance.ensemble.BalanceCascade,
                      ensemble.BalanceCascade)
        self.assertIs(df.imbalance.ensemble.EasyEnsemble,
                      ensemble.EasyEnsemble)

    def test_sample(self):
        from imblearn.under_sampling import ClusterCentroids, OneSidedSelection
        from imblearn.over_sampling import ADASYN, SMOTE
        from imblearn.combine import SMOTEENN, SMOTETomek

        models = [ClusterCentroids, OneSidedSelection, ADASYN, SMOTE,
                  SMOTEENN, SMOTETomek]

        X = np.random.randn(100, 5)
        y = np.array([0, 1]).repeat([80, 20])

        df = pdml.ModelFrame(X, target=y, columns=list('ABCDE'))

        for model in models:
            mod1 = model(random_state=self.random_state)
            mod2 = model(random_state=self.random_state)

            df.fit(mod1)
            mod2.fit(X, y)

            result = df.fit_resample(mod1)
            expected_X, expected_y = mod2.fit_resample(X, y)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_numpy_array_equal(result.target.values, expected_y)
            tm.assert_numpy_array_equal(result.data.values, expected_X)
            tm.assert_index_equal(result.columns, df.columns)

            mod1 = model(random_state=self.random_state)
            mod2 = model(random_state=self.random_state)

            result = df.fit_sample(mod1)
            expected_X, expected_y = mod2.fit_sample(X, y)

            self.assertIsInstance(result, pdml.ModelFrame)
            tm.assert_numpy_array_equal(result.target.values, expected_y)
            tm.assert_numpy_array_equal(result.data.values, expected_X)
            tm.assert_index_equal(result.columns, df.columns)

    def test_sample_ensemble(self):
        from imblearn.ensemble import BalanceCascade, EasyEnsemble

        models = [BalanceCascade, EasyEnsemble]

        X = np.random.randn(100, 5)
        y = np.array([0, 1]).repeat([80, 20])

        df = pdml.ModelFrame(X, target=y, columns=list('ABCDE'))

        for model in models:
            mod1 = model(random_state=self.random_state)
            mod2 = model(random_state=self.random_state)

            df.fit(mod1)
            mod2.fit(X, y)

            results = df.fit_resample(mod1)
            expected_X, expected_y = mod2.fit_resample(X, y)

            self.assertIsInstance(results, list)
            for r in results:
                self.assertIsInstance(r, pdml.ModelFrame)
                tm.assert_index_equal(r.columns, df.columns)

            mod1 = model(random_state=self.random_state)
            mod2 = model(random_state=self.random_state)

            results = df.fit_sample(mod1)
            expected_X, expected_y = mod2.fit_sample(X, y)

            self.assertIsInstance(results, list)
            for r in results:
                self.assertIsInstance(r, pdml.ModelFrame)
                tm.assert_index_equal(r.columns, df.columns)



#!/usr/bin/env python

import numpy as np
import sklearn.isotonic as isotonic

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestIsotonic(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.isotonic.IsotonicRegression, isotonic.IsotonicRegression)

    def test_isotonic_regression(self):
        data = np.abs(np.random.randn(100))
        data = data.cumsum()
        df = pdml.ModelFrame(np.arange(len(data)), target=data)

        result = df.isotonic.isotonic_regression()
        expected = isotonic.isotonic_regression(data)
        self.assertIsInstance(result, pdml.ModelSeries)
        tm.assert_index_equal(result.index, df.index)
        tm.assert_numpy_array_equal(result.values, expected)

    def test_check_increasing(self):
        data = np.abs(np.random.randn(100))
        data = data.cumsum()
        df = pdml.ModelFrame(np.arange(len(data)), target=data)

        result = df.isotonic.check_increasing()
        expected = isotonic.check_increasing(np.arange(len(data)), data)
        self.assertTrue(result)
        self.assertTrue(expected)

        data = np.abs(np.random.randn(100))
        data = data.cumsum()[-1::-1]        # reverse
        df = pdml.ModelFrame(np.arange(len(data)), target=data)

        result = df.isotonic.check_increasing()
        expected = isotonic.check_increasing(np.arange(len(data)), data)
        self.assertFalse(result)
        self.assertFalse(expected)

    def test_IsotonicRegression(self):
        # disable at this moment
        return
        """
        data = np.abs(np.random.randn(100))
        data = data.cumsum()
        df = pdml.ModelFrame(np.arange(len(data)), target=data)

        mod1 = df.isotonic.IsotonicRegression()
        mod2 = isotonic.IsotonicRegression()

        # df.fit(mod1)
        # mod2.fit(iris.data)

        # result = df.predict(mod1)
        # expected = mod2.predict(iris.data)

        # self.assertIsInstance(result, pdml.ModelSeries)
        # self.assert_numpy_array_almost_equal(result.values, expected)
        """



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods, _attach_methods, _wrap_data_target_func


class FeatureSelectionMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.feature_selection``.
    """

    _module_name = 'sklearn.feature_selection'


_fs_methods = ['chi2', 'f_classif', 'f_regression']
_attach_methods(FeatureSelectionMethods, _wrap_data_target_func, _fs_methods)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.covariance as covariance

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestCovariance(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.covariance.EmpiricalCovariance, covariance.EmpiricalCovariance)
        self.assertIs(df.covariance.EllipticEnvelope, covariance.EllipticEnvelope)
        self.assertIs(df.covariance.GraphLasso, covariance.GraphLasso)
        self.assertIs(df.covariance.GraphLassoCV, covariance.GraphLassoCV)
        self.assertIs(df.covariance.LedoitWolf, covariance.LedoitWolf)
        self.assertIs(df.covariance.MinCovDet, covariance.MinCovDet)
        self.assertIs(df.covariance.OAS, covariance.OAS)
        self.assertIs(df.covariance.ShrunkCovariance, covariance.ShrunkCovariance)

        self.assertIs(df.covariance.shrunk_covariance, covariance.shrunk_covariance)
        self.assertIs(df.covariance.graph_lasso, covariance.graph_lasso)

    def test_empirical_covariance(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.covariance.empirical_covariance()
        expected = covariance.empirical_covariance(iris.data)
        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.index, df.data.columns)
        tm.assert_index_equal(result.columns, df.data.columns)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_ledoit_wolf(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.covariance.ledoit_wolf()
        expected = covariance.ledoit_wolf(iris.data)

        self.assertEqual(len(result), 2)

        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.data.columns)
        tm.assert_index_equal(result[0].columns, df.data.columns)
        self.assert_numpy_array_almost_equal(result[0].values, expected[0])

        self.assert_numpy_array_almost_equal(result[1], expected[1])

    def test_oas(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.covariance.oas()
        expected = covariance.oas(iris.data)

        self.assertEqual(len(result), 2)

        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.data.columns)
        tm.assert_index_equal(result[0].columns, df.data.columns)
        self.assert_numpy_array_almost_equal(result[0].values, expected[0])

        self.assert_numpy_array_almost_equal(result[1], expected[1])

    @pytest.mark.parametrize("algo", ['EmpiricalCovariance', 'LedoitWolf'])
    def test_Covariance(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.covariance, algo)()
        mod2 = getattr(covariance, algo)()

        df.fit(mod1)
        mod2.fit(iris.data)

        self.assert_numpy_array_almost_equal(mod1.covariance_, mod2.covariance_)



#!/usr/bin/env python
import pytest

import matplotlib
matplotlib.use('Agg')

import numpy as np                            # noqa
import pandas as pd                           # noqa

import pandas_ml as pdml                      # noqa
import pandas_ml.util.testing as tm           # noqa

import sklearn.datasets as datasets           # noqa
import xgboost as xgb                         # noqa


class TestXGBoostPlotting(tm.TestCase):

    def test_plotting(self):

        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        df.fit(df.svm.SVC())

        # raises if df.estimator is not XGBModel
        with pytest.raises(ValueError):
            df.xgb.plot_importance()

        with pytest.raises(ValueError):
            df.xgb.to_graphviz()

        with pytest.raises(ValueError):
            df.xgb.plot_tree()

        df.fit(df.xgb.XGBClassifier())

        from matplotlib.axes import Axes
        from graphviz import Digraph

        try:
            ax = df.xgb.plot_importance()
        except ImportError:
            import nose
            # matplotlib.use doesn't work on Travis
            # PYTHON=3.4 PANDAS=0.17.1 SKLEARN=0.16.1
            raise nose.SkipTest()

        self.assertIsInstance(ax, Axes)
        assert ax.get_title() == 'Feature importance'
        assert ax.get_xlabel() == 'F score'
        assert ax.get_ylabel() == 'Features'
        assert len(ax.patches) == 4

        g = df.xgb.to_graphviz(num_trees=0)
        self.assertIsInstance(g, Digraph)

        ax = df.xgb.plot_tree(num_trees=0)
        self.assertIsInstance(ax, Axes)



#!/usr/bin/env python

import numpy as np
import pandas as pd

import sklearn.preprocessing as pp

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestModelSeries(tm.TestCase):

    def test_series_instance(self):
        s = pdml.ModelSeries([1, 2, 3, 4, 5], index=['A', 'B', 'C', 'D', 'E'])
        self.assertIsInstance(s, pdml.ModelSeries)

        s = s[['A', 'B']]
        self.assertEqual(len(s), 2)
        self.assertIsInstance(s, pdml.ModelSeries)

    def test_series_to_frame(self):
        s = pdml.ModelSeries([1, 2, 3, 4, 5])
        self.assertIsInstance(s, pdml.ModelSeries)

        df = s.to_frame()
        self.assertIsInstance(df, pdml.ModelFrame)
        tm.assert_index_equal(df.columns, pd.Index([0]))

        df = s.to_frame(name='x')
        self.assertIsInstance(df, pdml.ModelFrame)
        tm.assert_index_equal(df.columns, pd.Index(['x']))

        s = pdml.ModelSeries([1, 2, 3, 4, 5], name='name')
        self.assertIsInstance(s, pdml.ModelSeries)

        df = s.to_frame()
        self.assertIsInstance(df, pdml.ModelFrame)
        tm.assert_index_equal(df.columns, pd.Index(['name']))

        df = s.to_frame(name='x')
        self.assertIsInstance(df, pdml.ModelFrame)
        tm.assert_index_equal(df.columns, pd.Index(['x']))

    def test_preprocessing_normalize(self):
        s = pdml.ModelSeries([1, 2, 3, 4, 5], index=['A', 'B', 'C', 'D', 'E'])
        self.assertIsInstance(s, pdml.ModelSeries)
        result = s.preprocessing.normalize()
        expected = pp.normalize(np.atleast_2d(s.values.astype(np.float)))[0, :]

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)
        tm.assert_index_equal(result.index, s.index)



#!/usr/bin/env python
import pytest

import pandas as pd
import statsmodels.api as sm

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestStatsModelsDatasets(tm.TestCase):

    load_method = 'load'

    def test_anes96(self):
        data = getattr(sm.datasets.anes96, self.load_method)()
        df = pdml.ModelFrame(data)

        self.assertEqual(df.shape, (944, 6))
        self.assertEqual(df.target_name, 'PID')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_cancer(self):
        data = getattr(sm.datasets.cancer, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (301, 2))
        self.assertEqual(df.target_name, 'cancer')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_ccard(self):
        data = getattr(sm.datasets.ccard, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (72, 5))
        self.assertEqual(df.target_name, 'AVGEXP')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_committee(self):
        data = getattr(sm.datasets.committee, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (20, 6))
        self.assertEqual(df.target_name, 'BILLS104')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_copper(self):
        data = getattr(sm.datasets.copper, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (25, 6))
        self.assertEqual(df.target_name, 'WORLDCONSUMPTION')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_cpunish(self):
        data = getattr(sm.datasets.cpunish, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (17, 7))
        self.assertEqual(df.target_name, 'EXECUTIONS')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_elnino(self):
        data = getattr(sm.datasets.elnino, self.load_method)()
        msg = "Unable to read statsmodels Dataset without exog"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(data)

    def test_engel(self):
        data = getattr(sm.datasets.engel, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (235, 2))
        self.assertEqual(df.target_name, 'income')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_grunfeld(self):
        data = getattr(sm.datasets.grunfeld, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (220, 5))
        self.assertEqual(df.target_name, 'invest')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_longley(self):
        data = getattr(sm.datasets.longley, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (16, 7))
        self.assertEqual(df.target_name, 'TOTEMP')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_macrodata(self):
        data = getattr(sm.datasets.macrodata, self.load_method)()
        msg = "Unable to read statsmodels Dataset without exog"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(data)

    def test_modechoice(self):
        data = getattr(sm.datasets.modechoice, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (840, 7))
        self.assertEqual(df.target_name, 'choice')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_nile(self):
        data = getattr(sm.datasets.nile, self.load_method)()
        msg = "Unable to read statsmodels Dataset without exog"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(data)

    def test_randhie(self):
        data = getattr(sm.datasets.randhie, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (20190, 10))
        self.assertEqual(df.target_name, 'mdvis')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_scotland(self):
        data = getattr(sm.datasets.scotland, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (32, 8))
        self.assertEqual(df.target_name, 'YES')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_spector(self):
        data = getattr(sm.datasets.spector, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (32, 4))
        self.assertEqual(df.target_name, 'GRADE')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_stackloss(self):
        data = getattr(sm.datasets.stackloss, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (21, 4))
        self.assertEqual(df.target_name, 'STACKLOSS')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_star98(self):
        data = getattr(sm.datasets.star98, self.load_method)()
        msg = 'Data must be 1-dimensional'
        with pytest.raises(Exception, match=msg):
            pdml.ModelFrame(data)

    def test_strikes(self):
        data = getattr(sm.datasets.strikes, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (62, 2))
        self.assertEqual(df.target_name, 'duration')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_sunspots(self):
        data = getattr(sm.datasets.sunspots, self.load_method)()
        msg = "Unable to read statsmodels Dataset without exog"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(data)

    def test_fair(self):
        data = getattr(sm.datasets.fair, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (6366, 9))
        self.assertEqual(df.target_name, 'affairs')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_heart(self):
        data = getattr(sm.datasets.heart, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (69, 3))
        self.assertEqual(df.target_name, 'survival')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    def test_statecrime(self):
        data = getattr(sm.datasets.statecrime, self.load_method)()
        df = pdml.ModelFrame(data)
        self.assertEqual(df.shape, (51, 5))
        self.assertEqual(df.target_name, 'murder')
        tm.assert_index_equal(df.data.columns, pd.Index(data.exog_name))

    @pytest.mark.skip('skip until statsmodels GH#4775 is fixed')
    def test_co2(self):
        # https://github.com/statsmodels/statsmodels/issues/4775
        data = getattr(sm.datasets.co2, self.load_method)()
        msg = "Unable to read statsmodels Dataset without exog"
        with pytest.raises(ValueError, match=msg):
            pdml.ModelFrame(data)


class TestStatsModelsDatasets_LoadPandas(TestStatsModelsDatasets):

    load_method = 'load_pandas'

    def test_star98(self):
        data = sm.datasets.star98.load_pandas()
        if pdml.compat._PANDAS_ge_023:
            msg = 'Wrong number of items passed 2, placement implies 303'
        else:
            msg = 'cannot copy sequence with size 2 to array axis with dimension 303'
        with pytest.raises(Exception, match=msg):
            pdml.ModelFrame(data)



#!/usr/bin/env python

import pandas as pd

from pandas_ml.core import base


def _maybe_statsmodels_data(data, target):
    try:
        import statsmodels.datasets as datasets
        if isinstance(data, datasets.utils.Dataset):
            if target is not None:
                raise ValueError("'target' can't be specified for sklearn.datasets")

            # this should be first
            try:
                target_name = getattr(data, 'endog_name', None)
                target = pd.Series(data.endog, name=target_name)
            except AttributeError:
                target = None
            try:
                columns = getattr(data, 'exog_name', None)
                if hasattr(data, 'censors'):
                    data = {columns[0]: data.censors, columns[1]: data.exog}
                    # hack for "heart" dataset
                    data = pd.DataFrame(data, columns=columns)
                else:
                    data = pd.DataFrame(data.exog, columns=columns)
            except AttributeError:
                raise ValueError("Unable to read statsmodels Dataset without exog")

            return data, target
    except ImportError:
        pass

    return data, target


class StatsModelsRegressor(base._BaseEstimator, base._RegressorMixin):

    def __init__(self, statsmodel=None, **parameters):
        self.statsmodel = statsmodel
        self.parameters = parameters

    def fit(self, X, y, *args, **kwargs):
        if self.statsmodel is None:
            import statsmodels.api as sm
            self.statsmodel = sm.OLS

        statsmodel = self.statsmodel(y, X, **self.parameters)
        self.fitted_ = statsmodel.fit(*args, **kwargs)
        return self.fitted_

    def predict(self, X, *args, **kwargs):
        try:
            return self.fitted_.predict(X, *args, **kwargs)
        except AttributeError:
            raise ValueError('{0} is not fitted to data'.format(self.__class__.__name__))

    def get_params(self, deep=True):
        # suppose this estimator has parameters "alpha" and "recursive"
        params = self.parameters.copy()
        params['statsmodel'] = self.statsmodel
        return params

    def set_params(self, **parameters):
        if 'statsmodel' in parameters:
            self.statsmodel = parameters.pop('statsmodel')
        try:
            # updated existing parameters
            self.parameters.update(parameters)
        except AttributeError:
            # parameters may not exists in case of grid search
            self.parameters = parameters
        # must return itself to make grid_search work
        return self



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.kernel_approximation as ka

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestKernelApproximation(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.kernel_approximation.AdditiveChi2Sampler,
                      ka.AdditiveChi2Sampler)
        self.assertIs(df.kernel_approximation.Nystroem, ka.Nystroem)
        self.assertIs(df.kernel_approximation.RBFSampler, ka.RBFSampler)
        self.assertIs(df.kernel_approximation.SkewedChi2Sampler,
                      ka.SkewedChi2Sampler)

    @pytest.mark.parametrize("algo", ['AdditiveChi2Sampler'])
    def test_Classifications(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.kernel_approximation, algo)()
        mod2 = getattr(ka, algo)()

        df.fit(mod1)
        mod2.fit(iris.data)

        result = df.transform(mod1)
        expected = mod2.transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)

    @pytest.mark.parametrize("algo", ['Nystroem', 'RBFSampler',
                                      'SkewedChi2Sampler'])
    def test_Classifications_Random(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.kernel_approximation, algo)(random_state=self.random_state)
        mod2 = getattr(ka, algo)(random_state=self.random_state)

        df.fit(mod1)
        mod2.fit(iris.data)

        result = df.transform(mod1)
        expected = mod2.transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)



#!/usr/bin/env python

import importlib

import pandas.compat as compat


class _AccessorMethods(object):
    """
    Accessor to related functionalities.
    """

    _module_name = None
    _method_mapper = None

    def __init__(self, df, module_name=None, attrs=None):
        self._df = df

        if module_name is not None:
            # overwrite if exists
            self._module_name = module_name

        if self._module_name is None:
            return

        self._module = importlib.import_module(self._module_name)

        if attrs is None:
            try:
                attrs = self._module_attrs
            except AttributeError:
                pass
        if attrs is None:
            try:
                attrs = self._module.__all__
            except AttributeError:
                pass
        if attrs is None:
            return

        for mobj in attrs:
            try:
                if not hasattr(self, mobj):
                    try:
                        setattr(self, mobj, getattr(self._module, mobj))
                    except AttributeError:
                        pass

            except NotImplementedError:
                pass

    @property
    def _data(self):
        return self._df.data

    @property
    def _target(self):
        return self._df.target

    @property
    def _predicted(self):
        return self._df.predicted

    @property
    def _decision(self):
        return self._df.decision

    @property
    def _constructor(self):
        return self._df._constructor

    @property
    def _constructor_sliced(self):
        return self._df._constructor_sliced

    @classmethod
    def _update_method_mapper(cls, mapper):
        """Attach cls._method_mapper to passed mapper"""
        if cls._method_mapper is None:
            return mapper
        for key, class_dict in compat.iteritems(cls._method_mapper):
            # mapping method_name to actual class method
            class_dict = {k: getattr(cls, m) for k, m in compat.iteritems(class_dict)}
            mapper[key] = dict(mapper[key], **class_dict)
        return mapper


def _attach_methods(cls, wrap_func, methods):
    try:
        module = importlib.import_module(cls._module_name)

        for method in methods:
            _f = getattr(module, method)
            if hasattr(cls, method):
                raise ValueError("{0} already has '{1}' method".format(cls, method))
            func_name = cls._module_name + '.' + method
            setattr(cls, method, wrap_func(_f, func_name))

    except ImportError:
        pass


def _wrap_data_func(func, func_name):
    """
    Wrapper to call func with data values
    """
    def f(self, *args, **kwargs):
        data = self._data
        result = func(data.values, *args, **kwargs)
        return result
    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """ % func_name)
    return f


def _wrap_data_target_func(func, func_name):
    """
    Wrapper to call func with data and target values
    """
    def f(self, *args, **kwargs):
        data = self._data
        target = self._target
        result = func(data.values, y=target.values, *args, **kwargs)
        return result
    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``y``: ``ModelFrame.target``
        """ % func_name)
    return f


def _wrap_target_pred_func(func, func_name):
    """
    Wrapper to call func with target and predicted values
    """
    def f(self, *args, **kwargs):
        result = func(self._target.values, self._predicted.values,
                      *args, **kwargs)
        return result
    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """ % func_name)
    return f


def _wrap_target_pred_noargs(func, func_name):
    """
    Wrapper to call func with target and predicted values, accepting
    no other arguments
    """
    def f(self):
        result = func(self._target.values, self._predicted.values)
        return result
    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """ % func_name)
    return f



#!/usr/bin/env python

import matplotlib
matplotlib.use('Agg')

import numpy as np                            # noqa
import pandas as pd                           # noqa

import pandas_ml as pdml                      # noqa
import pandas_ml.util.testing as tm           # noqa

import sklearn.datasets as datasets           # noqa
import xgboost as xgb                         # noqa


class TestXGBoost(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.xgboost.XGBRegressor, xgb.XGBRegressor)
        self.assertIs(df.xgboost.XGBClassifier, xgb.XGBClassifier)

    def test_XGBClassifier(self):

        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        models = ['XGBClassifier']
        for model in models:
            mod1 = getattr(df.xgboost, model)()
            mod2 = getattr(xgb, model)()

            df.fit(mod1)
            mod2.fit(iris.data, iris.target)

            result = df.predict(mod1)
            expected = mod2.predict(iris.data)

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

    def test_XGBRegressor(self):
        # http://scikit-learn.org/stable/auto_examples/plot_kernel_ridge_regression.html
        X = 5 * np.random.rand(1000, 1)
        y = np.sin(X).ravel()

        # Add noise to targets
        y[::5] += 3 * (0.5 - np.random.rand(X.shape[0] // 5))

        df = pdml.ModelFrame(data=X, target=y)

        models = ['XGBRegressor']
        for model in models:
            mod1 = getattr(df.xgboost, model)()
            mod2 = getattr(xgb, model)()

            df.fit(mod1)
            mod2.fit(X, y)

            result = df.predict(mod1)
            expected = mod2.predict(X)

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

            self.assertIsInstance(df.predicted, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(df.predicted.values, expected)

    def test_grid_search(self):
        tuned_parameters = [{'max_depth': [3, 4],
                             'n_estimators': [50, 100]}]

        df = pdml.ModelFrame(datasets.load_digits())
        cv = df.model_selection.GridSearchCV(df.xgb.XGBClassifier(), tuned_parameters, cv=5)

        with tm.RNGContext(1):
            df.fit(cv)

        result = df.model_selection.describe(cv)
        expected = pd.DataFrame(cv.cv_results_)
        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_frame_equal(result, expected)



#!/usr/bin/env python
import pytest

import numpy as np
import sklearn.datasets as datasets
import sklearn.manifold as manifold

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestManifold(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.manifold.LocallyLinearEmbedding,
                      manifold.LocallyLinearEmbedding)
        self.assertIs(df.manifold.Isomap, manifold.Isomap)
        self.assertIs(df.manifold.MDS, manifold.MDS)
        self.assertIs(df.manifold.SpectralEmbedding, manifold.SpectralEmbedding)
        self.assertIs(df.manifold.TSNE, manifold.TSNE)

    def test_locally_linear_embedding(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.manifold.locally_linear_embedding(3, 3)
        expected = manifold.locally_linear_embedding(iris.data, 3, 3)

        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.index)
        tm.assert_numpy_array_equal(result[0].values, expected[0])

        self.assertEqual(result[1], expected[1])

    def test_spectral_embedding(self):
        N = 10
        m = np.random.random_integers(50, 200, size=(N, N))
        m = (m + m.T) / 2

        df = pdml.ModelFrame(m)
        self.assert_numpy_array_almost_equal(df.data.values, m)

        result = df.manifold.spectral_embedding(random_state=self.random_state)
        expected = manifold.spectral_embedding(m, random_state=self.random_state)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.index, df.index)
        # signs can be inversed
        self.assert_numpy_array_almost_equal(np.abs(result.data.values),
                                             np.abs(expected))

    @pytest.mark.parametrize("algo", ['Isomap'])
    def test_Isomap(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.manifold, algo)()
        mod2 = getattr(manifold, algo)()

        df.fit(mod1)
        mod2.fit(iris.data)

        result = df.transform(mod1)
        expected = mod2.transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.index, df.index)
        self.assert_numpy_array_almost_equal(result.data.values, expected)

    @pytest.mark.parametrize("algo", ['MDS'])
    def test_MDS(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.manifold, algo)(random_state=self.random_state)
        mod2 = getattr(manifold, algo)(random_state=self.random_state)

        result = df.fit_transform(mod1)
        expected = mod2.fit_transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.index, df.index)
        self.assert_numpy_array_almost_equal(result.data.values, expected)

    @pytest.mark.parametrize("algo", ['TSNE'])
    def test_TSNE(self, algo):
        digits = datasets.load_digits()
        df = pdml.ModelFrame(digits)

        mod1 = getattr(df.manifold, algo)(n_components=2, random_state=self.random_state)
        mod2 = getattr(manifold, algo)(n_components=2, random_state=self.random_state)

        # np.random.seed(1)
        result = df.fit_transform(mod1)
        # np.random.seed(1)
        expected = mod2.fit_transform(digits.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.index, df.index)
        self.assert_numpy_array_almost_equal(result.data.shape, expected.shape)
